@@ -7,16 +7,19 @@
 
 #include <gtk/gtk.h>
 #include "cfg.h"
-#include "edvtypes.h"
 #include "toolbar.h"
-#include "mountbar.h"
-#include "findbar.h"
-#include "statusbar.h"
-#include "fopdlg.h"
+#include "edv_types.h"
+#include "edv_mime_type.h"
+#include "edv_mount_bar.h"
+#include "edv_find_bar.h"
+#include "edv_status_bar.h"
+#include "obj_op_dlg.h"
 
 
-typedef struct _edv_browser_struct		edv_browser_struct;
-typedef struct _edv_browser_opid_struct		edv_browser_opid_struct;
+typedef struct _edv_browser_struct	edv_browser_struct;
+#define EDV_BROWSER(p)			((edv_browser_struct *)(p))
+typedef struct _edv_browser_opid_struct	edv_browser_opid_struct;
+#define EDV_BROWSER_OPID(p)		((edv_browser_opid_struct *)(p))
 
 
 /*
@@ -51,6 +54,8 @@ typedef enum {
  *	on the edv_browser_struct.
  */
 typedef enum {
+	EDV_BROWSER_CELL_STYLE_UNKNOWN,
+	EDV_BROWSER_CELL_STYLE_ERROR,
 	EDV_BROWSER_CELL_STYLE_NO_ACCESS,
 	EDV_BROWSER_CELL_STYLE_DANGLING_LINK,
 	EDV_BROWSER_CELL_STYLE_RECURSIVE_LINK,
@@ -58,7 +63,7 @@ typedef enum {
 	EDV_BROWSER_CELL_STYLE_EXECUTABLE
 } edv_browser_cell_style;
 /* Total number of cell styles */
-#define EDV_BROWSER_TOTAL_CELL_STYLES		5
+#define EDV_BROWSER_TOTAL_CELL_STYLES		7
 
 
 /*
@@ -77,6 +82,8 @@ typedef enum {
 	EDV_BROWSER_OP_RUN			= 303,
 	EDV_BROWSER_OP_RUN_TERMINAL		= 304,
 	EDV_BROWSER_OP_WRITE_PROTECT    	= 305,
+	EDV_BROWSER_OP_DELETE_METHOD_RECYCLE	= 308,
+	EDV_BROWSER_OP_DELETE_METHOD_PURGE	= 309,
 
 	EDV_BROWSER_OP_NEW			= 400,
 	EDV_BROWSER_OP_NEW_FILE			= 410,
@@ -88,7 +95,8 @@ typedef enum {
 	EDV_BROWSER_OP_NEW_SOCKET		= 417,
 
 	EDV_BROWSER_OP_OPEN			= 500,
-	EDV_BROWSER_OP_OPEN_WITH		= 501,
+	EDV_BROWSER_OP_OPEN_TO			= 501,
+	EDV_BROWSER_OP_OPEN_WITH		= 502,
 
 	EDV_BROWSER_OP_COPY_PATH		= 601,
 	EDV_BROWSER_OP_COPY_URL			= 602,
@@ -100,8 +108,9 @@ typedef enum {
 	EDV_BROWSER_OP_RENAME			= 703,
 	EDV_BROWSER_OP_CHMOD			= 704,
 	EDV_BROWSER_OP_CHOWN			= 705,
-	EDV_BROWSER_OP_DELETE			= 706,
-	EDV_BROWSER_OP_PROPERTIES		= 707,
+	EDV_BROWSER_OP_CHTIME			= 706,
+	EDV_BROWSER_OP_DELETE			= 770,
+	EDV_BROWSER_OP_PROPERTIES		= 780,
 
 	EDV_BROWSER_OP_SELECT_ALL		= 800,
 	EDV_BROWSER_OP_UNSELECT_ALL		= 801,
@@ -122,8 +131,9 @@ typedef enum {
 	EDV_BROWSER_OP_SHOW_FIND_BAR		= 1303,
 	EDV_BROWSER_OP_SHOW_STATUS_BAR		= 1304,
 
-	EDV_BROWSER_OP_SHOW_HIDDEN_OBJECTS	= 1400,
-	EDV_BROWSER_OP_SHOW_NOACCESS_OBJECTS	= 1401,
+	EDV_BROWSER_OP_CONTENTS_LIST_FILTER	= 1400,
+	EDV_BROWSER_OP_SHOW_HIDDEN_OBJECTS	= 1420,
+	EDV_BROWSER_OP_SHOW_NOACCESS_OBJECTS	= 1421,
 
 	EDV_BROWSER_OP_DIRECTORY_TREE_ORIGIN	= 1500,
 
@@ -198,7 +208,6 @@ struct _edv_browser_opid_struct {
 		**menu_icon_data;	/* Shared */
 
 };
-#define EDV_BROWSER_OPID(p)	((edv_browser_opid_struct *)(p))
 
 
 /*
@@ -210,7 +219,7 @@ struct _edv_browser_struct {
 	gboolean	processing;
 	gint		busy_count,
 			freeze_count;
-	gpointer	core_ptr;
+	edv_core_struct	*core;
 
 	GList		*opid_list;
 
@@ -245,16 +254,18 @@ struct _edv_browser_struct {
 	toolbar_struct	*tool_bar;
 
 	/* Mount Bar */
-	edv_mountbar_struct	*mount_bar;
+	edv_mount_bar_struct	*mount_bar;
 
 	/* Find Bar */
 	edv_findbar_struct	*find_bar;
 
 	/* Menus and Menu Items */
 	GtkWidget	*open_mi,
+			*open_to_mi,
+			*open_to_menu,
 			*open_with_mi,
-			*new_object_submi,
-			*new_object_submenu,
+			*new_object_mi,
+			*new_object_menu,
 			*new_file_mi,
 			*new_directory_mi,
 			*new_link_mi,
@@ -273,6 +284,7 @@ struct _edv_browser_struct {
 			*rename_mi,
 			*chmod_mi,
 			*chown_mi,
+			*chtime_mi,
 			*delete_mi,
 			*copy_path_mi,
 			*copy_url_mi,
@@ -297,12 +309,15 @@ struct _edv_browser_struct {
 			*show_mount_bar_micheck,
 			*show_find_bar_micheck,
 			*show_status_bar_micheck,
+			*contents_list_filter_mi,
 			*show_hidden_objects_micheck,
 			*show_noaccess_objects_micheck,
 			*directory_tree_origin_mi,
 			*mime_types_mi,
 
 			*write_protect_mi,
+			*delete_method_recycle_micheck,
+			*delete_method_purge_micheck,
 			*options_mi,
 			*customize_mi,
 
@@ -332,6 +347,7 @@ struct _edv_browser_struct {
 			*directory_ctree_rename_mi,
 			*directory_ctree_chmod_mi,
 			*directory_ctree_chown_mi,
+			*directory_ctree_chtime_mi,
 			*directory_ctree_delete_mi,
 			*directory_ctree_copy_path_mi,
 			*directory_ctree_copy_url_mi,
@@ -341,6 +357,8 @@ struct _edv_browser_struct {
 	/* Contents List right-click menu */
 	GtkWidget       *contents_clist_menu,
 			*contents_clist_open_mi,
+			*contents_clist_open_to_mi,
+			*contents_clist_open_to_menu,
 			*contents_clist_open_with_mi,
 			*contents_clist_move_mi,
 			*contents_clist_copy_mi,
@@ -348,6 +366,7 @@ struct _edv_browser_struct {
 			*contents_clist_rename_mi,
 			*contents_clist_chmod_mi,
 			*contents_clist_chown_mi,
+			*contents_clist_chtime_mi,
 			*contents_clist_delete_mi,
 			*contents_clist_copy_path_mi,
 			*contents_clist_copy_url_mi,
@@ -355,6 +374,10 @@ struct _edv_browser_struct {
 			*contents_clist_properties_mi;
 
 
+	/* Contents list filter */
+	gchar		*contents_list_filter;
+
+
 	/* Styles for the Directory Tree & Contents List cells */
 	GtkStyle	*cell_style[EDV_BROWSER_TOTAL_CELL_STYLES];
 
@@ -404,14 +427,13 @@ struct _edv_browser_struct {
 
 
 	/* Status Bar */
-	edv_statusbar_struct	*status_bar;
+	edv_status_bar_struct	*status_bar;
 
 
-	/* File Operation Dialog */
-	edv_fopdlg_struct	*fopdlg;
+	/* Object Operations Dialog */
+	edv_obj_op_dlg_struct	*obj_op_dlg;
 
 };
-#define EDV_BROWSER(p)	((edv_browser_struct *)(p))
 
 
 /* Utilities */
@@ -419,7 +441,7 @@ extern edv_browser_opid_struct *EDVBrowserMatchOPID(
 	edv_browser_struct *browser, edv_browser_op op
 );
 extern gboolean EDVBrowserIsPathFromDirTreeOrigin(
-        edv_browser_struct *browser, const gchar *path
+	edv_browser_struct *browser, const gchar *path
 );
 
 /* Title */
@@ -443,6 +465,9 @@ extern void EDVBrowserAccelkeysRegenerate(edv_browser_struct *browser);
 extern void EDVBrowserListStylesRegenerate(edv_browser_struct *browser);
 extern void EDVBrowserToolBarRegenerate(edv_browser_struct *browser);
 extern void EDVBrowserNewObjectMenuRegenerate(edv_browser_struct *browser);
+extern void EDVBrowserOpenToMenuRegenerate(
+	edv_browser_struct *browser, edv_mime_type_struct *m
+);
 extern void EDVBrowserDirectoryMenuRegenerate(edv_browser_struct *browser);
 extern void EDVBrowserContentsMenuRegenerate(edv_browser_struct *browser);
 
@@ -459,7 +484,7 @@ extern void EDVBrowserOPIDListDelete(GList *glist);
 extern cfg_accelkey_list_struct *EDVBrowserAccelkeysListNewDefault(void);
 
 /* File Browser */
-extern edv_browser_struct *EDVBrowserNew(gpointer core_ptr);
+extern edv_browser_struct *EDVBrowserNew(edv_core_struct *core);
 extern void EDVBrowserSyncData(edv_browser_struct *browser);
 extern void EDVBrowserUpdateMenus(edv_browser_struct *browser);
 extern void EDVBrowserSetBusy(edv_browser_struct *browser, gboolean busy);
