@@ -1,7 +1,7 @@
 /*
                    Endeavour Compile Time Configuration
 
-	See edvcfglist.h for confirmation list.
+	See edv_cfg_list.h for confirmation list.
  */
 
 #ifndef CONFIG_H
@@ -25,34 +25,34 @@
 /*
  *      Program name and version:
  */
-#define PROG_NAME		"Endeavour"
-#define PROG_NAME_FULL		"Endeavour Mark II"
-#define PROG_VERSION		"2.5.7"
+#define PROG_NAME			"Endeavour"
+#define PROG_NAME_FULL			"Endeavour Mark II"
+#define PROG_VERSION			"2.7.1"
 
-#define PROG_VERSION_MAJOR	2
-#define PROG_VERSION_MINOR	5
-#define PROG_VERSION_RELEASE	7
+#define PROG_VERSION_MAJOR		2
+#define PROG_VERSION_MINOR		7
+#define PROG_VERSION_RELEASE		1
 
 /*
  *      Copyright message:
  */
 #if defined(PROG_LANGUAGE_SPANISH)
 #define PROG_COPYRIGHT	"\
-El derecho de autor (C) 1997-2005 WolfPack Entertainment.\n\
+El derecho de autor (C) 1997-2006 WolfPack Entertainment.\n\
 Este programa es protegido por leyes internacionales de derecho de autor\n\
 y tratados, la distribucin y/o la modificacin de este software en la\n\
 infraccin de la Licencia del Pblico de GNU se prohiben estrictamente.\n\
 Violators ser procesado a la extensin ms repleta de la ley.\n"
 #elif defined(PROG_LANGUAGE_FRENCH)
 #define PROG_COPYRIGHT  "\
-Dposer (C) 1997-2005 WolfPack Entertainment.\n\
+Dposer (C) 1997-2006 WolfPack Entertainment.\n\
 Ce programme est protg par international dpose lois et traits, \n\
 distribution et/ou de ce logiciel dans violation du GNU Public Permis\n\
 strictement est interdit. Violators sera poursuivi  l'tendue plus pleine\n\
 de la loi.\n"
 #elif defined(PROG_LANGUAGE_GERMAN)
 #define PROG_COPYRIGHT  "\
-Urheberrecht (C) 1997-2005 WolfPack Entertainment.\n\
+Urheberrecht (C) 1997-2006 WolfPack Entertainment.\n\
 Dieses Programm wird von internationalem Urheberrecht Gesetzen und\n\
 Vertrgen, Austeilung bzw. nderung dieser Software in bertretung\n\
 DER GNU ffentlichkeit Erlaubnis streng verboten wird geschtzt.\n\
@@ -60,35 +60,35 @@ Verletzer werden zum vollsten Umfang des Gesetzes strafrechtlich\n\
 verfolgt werden.\n"
 #elif defined(PROG_LANGUAGE_ITALIAN)
 #define PROG_COPYRIGHT  "\
-Il copyright (C) 1997-2005 WolfPack Entertainment.\n\
+Il copyright (C) 1997-2006 WolfPack Entertainment.\n\
 Questo programma  protetto dalle leggi di copyright internazionali e\n\
 dai trattati, la distribuzione e/o la modifica di questo software nella\n\
 violazione dello GNU la Licenza Pubblica  strettamente proibita. Il\n\
 Violators sar perseguito all'estensione la pi piena della legge.\n"
 #elif defined(PROG_LANGUAGE_DUTCH)
 #define PROG_COPYRIGHT  "\
-Auteursrecht (C) 1997-2005 WolfPack Entertainment.\n\
+Auteursrecht (C) 1997-2006 WolfPack Entertainment.\n\
 Deze programma wordt door internationale auteursrecht wetten en\n\
 verdragen, verdeling en/of wijziging van deze software in overtreding\n\
 van de GNU Publiek Vergunning strikt verboden wordt beschermd.\n\
 Overtreders zullen te de volste omvang van de wet geprocedeerd worden.\n"
 #elif defined(PROG_LANGUAGE_PORTUGUESE)
 #define PROG_COPYRIGHT  "\
-Os direitos autorais (C) 1997-2005 WolfPack Entertainment.\n\
+Os direitos autorais (C) 1997-2006 WolfPack Entertainment.\n\
 Este programa  protegido por direitos internacionais leis autorais e\n\
 tratados, modificao de e/ou de distribuio deste software em infrao\n\
 do GNU Licena Pblica precisamente  proibida. O Violators ser\n\
 processado  plena extenso da lei.\n"
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 #define PROG_COPYRIGHT  "\
-Copyright (C) 1997-2005 WolfPack Entertainment.\n\
+Copyright (C) 1997-2006 WolfPack Entertainment.\n\
 Dette programet beskytter av internasjonal copyrightlover og traktater,\n\
 distribusjon og/eller modifikasjon av denne programvaren i overtredelse\n\
 av GNU Public License er streng forbudt. Violators tiltalt til den\n\
 fulleste omfang av loven.\n"
 #else
 #define PROG_COPYRIGHT  "\
-Copyright (C) 1997-2005 WolfPack Entertainment.\n\
+Copyright (C) 1997-2006 WolfPack Entertainment.\n\
 This program is protected by international copyright laws and treaties,\n\
 distribution and/or modification of this software in violation of the\n\
 GNU Public License is strictly prohibited. Violators will be prosecuted\n\
@@ -135,10 +135,18 @@ to the fullest extent of the law.\n"
 #define ENV_VAR_NAME_HELPBROWSER		"HELPBROWSER"
 #define ENV_VAR_NAME_HOME			"HOME"
 #define ENV_VAR_NAME_PATH			"PATH"
+#define ENV_VAR_NAME_PRINT			"PRINT"
+#define ENV_VAR_NAME_PRINTCMD			"PRINTCMD"
+#define ENV_VAR_NAME_PRINTCOMMAND		"PRINTCOMMAND"
+#define ENV_VAR_NAME_PRINTER			"PRINTER"
 #define ENV_VAR_NAME_TMPDIR			"TMPDIR"
 #define ENV_VAR_NAME_USER			"USER"
 #define ENV_VAR_NAME_USERNAME			"USERNAME"
 
+#define ENV_VAR_NAME_EDVCFGFILE			"EDVCFGFILE"
+#define ENV_VAR_NAME_EDVINTERPSLOCK		"EDVINTERPSLOCK"
+#define ENV_VAR_NAME_EDVINTERPSCMDFILE		"EDVINTERPSCMDFILE"
+
 
 /*
  *	GTK Font Names:
@@ -237,6 +245,7 @@ to the fullest extent of the law.\n"
 	".tar.bz2",	"BZip2 Compressed UNIX Tape Archive",	\
 	".tar.gz .tgz",	"GZip Compressed UNIX Tape Archive",	\
 	".tar.Z",	"Compressed UNIX Tape Archive",		\
+	".xar",		"X Archive",				\
 	".xpi",		"PKZip Archive With Java Script",	\
 	".zip",		"PKZip Archive",			\
 	NULL,		NULL					\
@@ -404,6 +413,13 @@ to the fullest extent of the law.\n"
  */
 #define EDV_DEF_PROGRESS_BAR_TICKS	25
 
+/*
+ *	Default Progress Dialog Animation Interval & Animation
+ *	Increment:
+ */
+#define EDV_DEF_PROGRESS_DLG_ANIM_INT	500l
+#define EDV_DEF_PROGRESS_DLG_ANIM_INC	((guint16)10000)
+
 
 /*
  *	Default File Browser Geometry:
@@ -688,6 +704,7 @@ El uso: endeavour2 [path] [options] [GUI_options]\n\
         --devices               Arranque con el Devices List.\n\
         --mimetypes             Arranque con el MIME Types List.\n\
         --run-dialog            Arranque con el Run Dialog.\n\
+        --find                  Arranque con el Find Window.\n\
         --about-dialog          Arranque con el About Dialog.\n\
         --help                  La pantalla de la ayuda de impresiones\n\
                                 (esto) y salidas.\n\
@@ -729,6 +746,7 @@ Usage: endeavour2 [path] [options] [GUI_options]\n\
         --devices               Le dmarrage avec le Devices List.\n\
         --mimetypes             Le dmarrage avec le MIME Types List.\n\
         --run-dialog            Le dmarrage avec le Run Dialog.\n\
+        --find                  Le dmarrage avec le Find Window.\n\
         --about-dialog          Le dmarrage avec le About Dialog.\n\
         --help                  Caractres (ceci) aide cran et sorties.\n\
         --version               Caractres information de version et\n\
@@ -746,18 +764,23 @@ Usage: endeavour2 [path] [options] [GUI_options]\n\
 \n"
 #else
 #define PROG_HELP_MESG  "\
-Usage: endeavour2 [path] [options] [GUI_options]\n\
+Usage: endeavour2 [path] [extra] [options] [GUI_options]\n\
 \n\
-    [path] specifies the path to an alternate startup location.\n\
+    The [path] specifies the path of the startup location.\n\
 \n\
-    [options] can be any of the following:\n\
+    The [extra] specifies the page, password, second path, or\n\
+    subsection based on the window that was specified at startup.\n\
+\n\
+    The [options] can be any of the following:\n\
 \n\
         --config <file>         Load configuration from <file>.\n\
         --rcfile                Same as --config.\n\
         -f                      Same as --config.\n\
+\n\
         --no-splash             Do not show splash on startup.\n\
         --no-stdout-dialog      Do not show stdout messages in a dialog.\n\
         --no-stderr-dialog      Do not show stderr messages in a dialog.\n\
+\n\
         --file-browser          Start up with the File Browser.\n\
         --image-browser         Start up with the Image Browser.\n\
         --archiver              Start up with the Archiver.\n\
@@ -767,12 +790,14 @@ Usage: endeavour2 [path] [options] [GUI_options]\n\
         --customize             Start up with the Customize Window.\n\
         --devices               Start up with the Devices List.\n\
         --mimetypes             Start up with the MIME Types List.\n\
-        --run-dialog            Start up with the Run Dialog.\n\
-        --about-dialog          Start up with the About Dialog.\n\
+        --run                   Start up with the Run Dialog.\n\
+        --find                  Start up with the Find Window.\n\
+        --about                 Start up with the About Dialog.\n\
+\n\
         --help                  Prints (this) help screen and exits.\n\
         --version               Prints version information and exits.\n\
 \n\
-    [GUI_options] can be any of the following\n\
+    The [GUI_options] can be any of the following\n\
 \n\
         --geometry <WxH+X+Y>    Specifies the geometry of the main windows.\n\
         --font <font>           Specifies the font to be used for the lists.\n\
@@ -798,9 +823,6 @@ Usage: endeavour2 [path] [options] [GUI_options]\n\
     windows (under the Settings menu).\n\
 \n\
     Command line options override any options in the configuration file.\n\
-\n\
-    [GUI_options] can be any options standard to your GUI, consult your\n\
-    GUI's manual for available options.\n\
 \n"
 #endif
 
