@@ -18,9 +18,9 @@
 #include "../fb.h"
 #include "../progressdialog.h"
 
-#include "../edvtypes.h"
-#include "../edvcfglist.h"
-#include "../edvutils.h"
+#include "../edv_types.h"
+#include "../edv_cfg_list.h"
+#include "../edv_utils.h"
 #include "../lib/endeavour2.h"
 
 #include "downloadcfgio.h"
@@ -41,7 +41,13 @@
 #include "../images/pdi_folderfile_32x32.xpm"
 
 
-static glong CurrentMillitime(void);
+static gulong DownloadCurrentMS(void);
+
+#if 0
+static gulong DownloadGetAvgFromTimeBuffer(
+	const gulong *buf, const gint nitems
+);
+#endif
 
 static void DownloadSignalCB(int s);
 static gchar *DownloadBrowseTargetPathCB(
@@ -50,9 +56,12 @@ static gchar *DownloadBrowseTargetPathCB(
 
 static gulong DownloadTryGetTotalLength(FILE *fp);
 static gint DownloadMonitor(
-	edv_context_struct *ctx, gint p,
+	edv_context_struct *ctx,
+	const gint p, const gulong check_interval_ms,
 	const gchar *src, const gchar *tar,
-	const gchar *stdout_file
+	const gchar *stdout_file,
+	const GdkGeometryFlags geometry_flags,
+	const GdkRectangle *geometry
 );
 
 
@@ -71,14 +80,14 @@ static gint DownloadMonitor(
 /*
  *      Returns the current time in milliseconds.
  */
-static glong CurrentMillitime(void)
+static gulong DownloadCurrentMS(void)
 {
 #ifdef __MSW__
 	SYSTEMTIME t;   /* Current time of day structure */
 
 	GetSystemTime(&t);
 	return(
-	    (glong)(
+	    (gulong)(
 		(((((t.wHour * 60.0) + t.wMinute) * 60) + t.wSecond) * 1000) +
 		t.wMilliseconds
 	    )
@@ -87,12 +96,43 @@ static glong CurrentMillitime(void)
 	struct timeval tv[1];
 
 	if(gettimeofday(tv, NULL) < 0)
-	    return(-1);
+	    return(0l);
 
-	return(((tv->tv_sec % 86400) * 1000) + (tv->tv_usec / 1000));
+	return((gulong)(
+	    ((tv->tv_sec % 86400l) * 1000l) + (tv->tv_usec / 1000l)
+	));
 #endif
 }
 
+#if 0
+/*
+ *	Returns the average value of the specified time buffer.
+ *
+ *	Note that this average is biased to the last (highest) index
+ *	value.
+ */
+static gulong DownloadGetAvgFromTimeBuffer(
+	const gulong *buf, const gint nitems 
+)
+{
+	gint i;
+	gulong l;
+
+	if((buf == NULL) || (nitems <= 0))
+	    return(0l);
+
+	if(nitems == 1)
+	    return(buf[0]);
+
+	l = buf[0];
+	for(i = 1; i < nitems; i++)
+	    l += buf[i];
+
+	return(l / nitems);
+}
+#endif
+
+
 /*
  *      UNIX signal callback.
  */
@@ -124,13 +164,13 @@ static gchar *DownloadBrowseTargetPathCB(
 	if(FileBrowserIsQuery())
 	    return(NULL);
 
-	/* Create file types list */
+	/* Create the file types list */
 	FileBrowserTypeListNew(
 	    &ftype, &total_ftypes,
 	    "*.*", "All files"
 	);
 
-	/* Query user for path */
+	/* Query the user for path */
 	FileBrowserSetTransientFor(toplevel);
 	status = FileBrowserGetResponse(
 #if defined(PROG_LANGUAGE_SPANISH)
@@ -293,25 +333,38 @@ static gulong DownloadTryGetTotalLength(FILE *fp)
  *	-4	User aborted
  */
 static gint DownloadMonitor(
-	edv_context_struct *ctx, gint p,
+	edv_context_struct *ctx,
+	const gint p, const gulong check_interval_ms,
 	const gchar *src, const gchar *tar,
-	const gchar *stdout_file
+	const gchar *stdout_file,
+	const GdkGeometryFlags geometry_flags,
+	const GdkRectangle *geometry
 )
 {
 	FILE *fp;
 	gboolean	notified_object_added = FALSE,
 			user_abort = FALSE;
 	const gchar *cstrptr;
+	const gint check_count_max = (gint)(
+	    check_interval_ms * 1000l / 20000l
+	);
 	gint check_count = 0, notify_count = 0;
-	gulong last_length = 0, current_length = 0, total_length = 0;
-	glong last_millitime = CurrentMillitime();
+
+#if 0
+gint time_left_sec_nitems = 0;
+gulong time_left_sec_buf[1];
+const int time_left_sec_max_items = sizeof(time_left_sec_buf) / sizeof(gulong);
+#endif
+	gulong	last_length = 0l,
+		current_length = 0l,
+		total_length = 0l,
+		last_millitime = DownloadCurrentMS();
 	gchar *title, *message, *p1, *p2, *tar_name;
 	guint8        **start_icon_data[3],
 			**icon_data[6],
 			**end_icon_data[3];
 	struct stat stat_buf;
 
-
 	/* Get just the name of the target */
 	cstrptr = strrchr(tar, '/');
 	if(cstrptr != NULL)
@@ -357,14 +410,16 @@ static gint DownloadMonitor(
 	    " %s", tar_name
 	);
 
-	/* Format source and target paths for message */
+	/* Format the source and target paths for the message, shorten
+	 * them so that the message on each line is fully visible
+	 */
 	p1 = EDVShortenPath(
-	    src, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX - 10
+	    src, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX - 6
 	);
 	p2 = EDVShortenPath(
-	    tar, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX - 10
+	    tar, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX - 16
 	);
-	/* Format initial message */
+	/* Format the initial message */
 	message = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Descargar:\n\
@@ -434,10 +489,28 @@ To:\n\
 	    , p1, p2
 	);
 
-	/* Map progress dialog */
+	/* Set the progress dialog's WM icon */
 	ProgressDialogSetWMIconData(
 	    (const guint8 **)icon_download_file_48x48_xpm
 	);
+
+	/* Set the progress dialog's geometry */
+	if(geometry != NULL)
+	{
+	    GtkWidget *w = ProgressDialogGetToplevel();
+	    if(w != NULL)
+	    {
+		gtk_widget_set_uposition(
+		    w,
+		    (geometry_flags & GDK_GEOMETRY_X) ?
+			geometry->x : 0,
+		    (geometry_flags & GDK_GEOMETRY_Y) ?
+			geometry->y : 0
+		);
+	    }
+	}
+
+	/* Map the progress dialog */
 	ProgressDialogMapAnimation(
 	    title,
 	    message,
@@ -471,7 +544,7 @@ To:\n\
 	message = NULL;
 
 
-	/* Open stdout file */
+	/* Open the download process' stdout file */
 	fp = FOpen(stdout_file, "rb");
 
 	/* While the download process is running... */
@@ -485,10 +558,15 @@ To:\n\
 		break;
 	    }
 
+#if 1
+	    /* Time to check on the download progress? */
+	    if(check_count >= check_count_max)
+#else
 	    /* Time to do some checks (at least every 1 second)? */
 	    if(check_count >= 50)
+#endif
 	    {
-		/* Check if download process is done */
+		/* Is the download process is done */
 		if(!ExecProcessExists(p))
 		    break;
 
@@ -503,48 +581,79 @@ To:\n\
 		if(total_length == 0)
 		    total_length = DownloadTryGetTotalLength(fp);
 
-		/* Make sure the target object exists and get its 
-		 * current length
-		 */
+		/* Get the stats of the target object */
 		if(!stat(tar, &stat_buf))
 		{
-		    glong current_millitime, delta_millitime;
-		    glong time_left_seconds;
-		    gulong delta_length, remaining_length;
 		    gchar time_left_str[80];
+		    gulong	cur_ms, delta_ms,
+				time_remaining_sec,
+				delta_length, remaining_length;
 
 		    /* Update current length and time lapse between
 		     * length checks
 		     */
 		    last_length = current_length;
-		    current_length = stat_buf.st_size;
-		    delta_length = current_length - last_length;
+		    current_length = (gulong)stat_buf.st_size;
+		    delta_length = (current_length > last_length) ?
+			(current_length - last_length) : 0l;
 		    remaining_length = total_length - current_length;
 
-		    current_millitime = CurrentMillitime();
-		    if((current_millitime > last_millitime) &&
-		       (last_millitime > 0)
+		    cur_ms = DownloadCurrentMS();
+		    if((cur_ms > last_millitime) &&
+		       (last_millitime > 0l)
 		    )
-			delta_millitime = current_millitime - last_millitime;
+			delta_ms = cur_ms - last_millitime;
 		    else
-			delta_millitime = 0;
-		    last_millitime = current_millitime;
+			delta_ms = 0l;
+		    last_millitime = cur_ms;
 
-		    /* Calculate time left in seconds, if the result
+		    /* Calculate the time left in seconds, if the result
 		     * is 0 then it means the transfer has stalled
 		     */
-		    if((total_length > 0) && (total_length != (gulong)-1) &&
-		       (delta_length > 0) && (remaining_length > 0)
+		    if((total_length > 0l) && (total_length != (gulong)-1) &&
+		       (delta_length > 0l) && (remaining_length > 0l)
 		    )
-			time_left_seconds = (glong)(
-			    (gfloat)delta_millitime / (gfloat)delta_length *
+			time_remaining_sec = (gulong)(
+			    (gfloat)delta_ms / (gfloat)delta_length *
 			    (gfloat)remaining_length / 1000.0f
 			);
 		    else
-			time_left_seconds = 0;
+			time_remaining_sec = 0l;
+
+#if 0
+/* The time left calculation is a bit "jumpy", this is most likely
+ * due to the fact that this process is unable to determine precisely,
+ * the moment, when additional bytes are downloaded to the target object
+ * in order to accurately calculate the delta_ms that is
+ * needed to predict how many bytes were downloaded during this
+ * interval and ultimately the time remaining
+ */
+		    /* Append this time left value to the time left
+		     * history buffer and then calculate the average
+		     * time left
+		     */
+		    if(time_remaining_sec > 0l)
+		    {
+			const gint	m = MIN(
+			    (time_left_sec_nitems + 1),
+			    time_left_sec_max_items
+			),
+					im = m - 1;
+			gint i;
+
+			for(i = 0; i < im; i++)
+			    time_left_sec_buf[i] = time_left_sec_buf[i + 1];
+			time_left_sec_buf[im] = time_remaining_sec;
+			time_left_sec_nitems = m;
+
+			time_remaining_sec = DownloadGetAvgFromTimeBuffer(
+			    time_left_sec_buf, time_left_sec_nitems
+			);
+		    }
+#endif
 
 		    /* Format time left string */
-		    if(time_left_seconds > 3600)
+		    if(time_remaining_sec > 3600l)
 			g_snprintf(
 			    time_left_str, sizeof(time_left_str),
 			    "%ld:%.2ld:%.2ld "
@@ -565,11 +674,11 @@ To:\n\
 #else
 "left"
 #endif
-			    , (time_left_seconds / 3600),
-			    (time_left_seconds / 60) % 60,
-			    (time_left_seconds / 1) % 60
+			    , (time_remaining_sec / 3600l),
+			    (time_remaining_sec / 60l) % 60l,
+			    (time_remaining_sec / 1l) % 60l
 			);
-		    else if(time_left_seconds > 0)
+		    else if(time_remaining_sec > 0l)
 			g_snprintf(
 			    time_left_str, sizeof(time_left_str),
 			    "%ld:%.2ld "
@@ -590,8 +699,8 @@ To:\n\
 #else
 "left"
 #endif
-			    , (time_left_seconds / 60),
-			    (time_left_seconds / 1) % 60
+			    , (time_remaining_sec / 60l),
+			    (time_remaining_sec / 1l) % 60l
 			);
 		    else
 			g_snprintf(
@@ -616,8 +725,11 @@ To:\n\
 			);
 
 #if 0
-g_print("Time left %ld seconds (%ld left) %ld %ld\n",
- time_left_seconds, remaining_length, current_length, total_length);
+g_print(
+ "Time left %ld seconds (%ld left) %ld %ld\n",
+ time_remaining_sec, remaining_length,
+ current_length, total_length
+);
 #endif
 
 		    /* Need to notify about the object being
@@ -632,9 +744,9 @@ g_print("Time left %ld seconds (%ld left) %ld %ld\n",
 			notified_object_added = TRUE;
 		    }
 
-		    /* Update title string */
+		    /* Update the title string */
 		    g_free(title);
-		    if((total_length > 0) && (total_length != (gulong)-1))
+		    if((total_length > 0l) && (total_length != (gulong)-1))
 			title = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Descargar"
@@ -680,9 +792,9 @@ g_print("Time left %ld seconds (%ld left) %ld %ld\n",
 			     " %s", tar_name
 			);
 
-		    /* Update message string */
+		    /* Update the message string */
 		    g_free(message);
-		    if((total_length > 0) && (total_length != (gulong)-1))
+		    if((total_length > 0l) && (total_length != (gulong)-1))
 		    {
 			gchar	*total_size_str = STRDUP(EDVSizeStrDelim(total_length)),
 				*cur_size_str = STRDUP(EDVSizeStrDelim(current_length));
@@ -844,7 +956,7 @@ To:\n\
 	    }
 
 	    /* Time to do notifies (every 10 seconds or longer)? */
-	    if(notify_count >= 500)
+	    if(notify_count >= 500l)
 	    {
 		if(notified_object_added)
 		{
@@ -895,7 +1007,7 @@ To:\n\
 		message = NULL;
 	    }
 
-	    usleep(20000);
+	    usleep(20000l);
 	}
 
 	/* Need to unmap progress dialog just in case */
@@ -1190,8 +1302,11 @@ int main(int argc, char *argv[])
 			beep_when_complete = FALSE,
 			open_object = FALSE,
 			download_last_object = FALSE;
+	gulong		check_interval_ms = 1000l;
+	GdkGeometryFlags geometry_flags = 0;
+	GdkRectangle *geometry = NULL;
 	gint i, status = 0;
-	const gchar *arg_ptr;
+	const gchar *arg;
 	gchar *prog = NULL;
 	gchar *source_url = NULL;
 	gchar *destination_path = NULL;
@@ -1227,16 +1342,17 @@ int main(int argc, char *argv[])
   PDialogShutdown();			\
   FileBrowserShutdown();		\
   ProgressDialogShutdown();		\
+					\
+  /* Reset the DND Icon */		\
+  GUIDNDSetDragIcon(NULL, NULL, 0, 0);	\
+					\
   initialized_gtk = FALSE;		\
  }					\
 					\
- /* Reset the DND Icon */		\
- GUIDNDSetDragIcon(NULL, NULL, 0, 0);	\
+ g_free(geometry);			\
+ geometry = NULL;			\
 }
 
-	/* Set up time zone */
-	tzset();
-
 	/* Set up signal callbacks */
 #ifdef SIGINT
 	signal(SIGINT, DownloadSignalCB);
@@ -1264,16 +1380,16 @@ int main(int argc, char *argv[])
 	/* Parse arguments */
 	for(i = 1; i < argc; i++)
 	{
-	    arg_ptr = argv[i];
-	    if(arg_ptr == NULL)
+	    arg = argv[i];
+	    if(arg == NULL)
 		continue;
 
 	    /* Help */
-	    if(!g_strcasecmp(arg_ptr, "--help") ||
-	       !g_strcasecmp(arg_ptr, "-help") ||
-	       !g_strcasecmp(arg_ptr, "--h") ||
-	       !g_strcasecmp(arg_ptr, "-h") ||
-	       !g_strcasecmp(arg_ptr, "-?")
+	    if(!g_strcasecmp(arg, "--help") ||
+	       !g_strcasecmp(arg, "-help") ||
+	       !strcmp(arg, "--h") ||
+	       !strcmp(arg, "-h") ||
+	       !strcmp(arg, "-?")
 	    )
 	    {
 		g_print("%s", PROG_HELP_MESG);
@@ -1282,10 +1398,8 @@ int main(int argc, char *argv[])
 		return(status);
 	    }
 	    /* Version */
-	    else if(!g_strcasecmp(arg_ptr, "--version") ||
-		    !g_strcasecmp(arg_ptr, "-version") ||
-		    !g_strcasecmp(arg_ptr, "--v") ||
-		    !g_strcasecmp(arg_ptr, "-v")
+	    else if(!g_strcasecmp(arg, "--version") ||
+		    !g_strcasecmp(arg, "-version")
 	    )
 	    {
 		g_print("%s %s\n%s", PROG_NAME, PROG_VERSION, PROG_COPYRIGHT);
@@ -1293,50 +1407,163 @@ int main(int argc, char *argv[])
 		DO_SHUTDOWN
 		return(status);
 	    }
-	    /* Beep when download is complete? */
-	    else if(!g_strcasecmp(arg_ptr, "--beep") ||
-		    !g_strcasecmp(arg_ptr, "-beep") ||
-		    !g_strcasecmp(arg_ptr, "--b") ||
-		    !g_strcasecmp(arg_ptr, "-b")
+	    /* Beep when the download has been complete? */
+	    else if(!g_strcasecmp(arg, "--beep") ||
+		    !g_strcasecmp(arg, "-beep") ||
+		    !strcmp(arg, "--b") ||
+		    !strcmp(arg, "-b")
 	    )
 	    {
 		beep_when_complete = TRUE;
 	    }
-	    /* Need confirmation? */
-	    else if(!g_strcasecmp(arg_ptr, "--confirm") ||
-		    !g_strcasecmp(arg_ptr, "-confirm") ||
-		    !g_strcasecmp(arg_ptr, "--c") ||
-		    !g_strcasecmp(arg_ptr, "-c")
+	    /* Query the user for confirmation before downloading? */
+	    else if(!g_strcasecmp(arg, "--confirm") ||
+		    !g_strcasecmp(arg, "-confirm") ||
+		    !strcmp(arg, "--c") ||
+		    !strcmp(arg, "-c")
 	    )
 	    {
 		need_confirmation = TRUE;
 	    }
-	    /* Open object after (successful and complete) download? */
-	    else if(!g_strcasecmp(arg_ptr, "--open") ||
-		    !g_strcasecmp(arg_ptr, "-open") ||
-		    !g_strcasecmp(arg_ptr, "--o") ||
-		    !g_strcasecmp(arg_ptr, "-o")
+	    /* Open the object afterwards (only if the download was
+	     * successful and complete)?
+	     */
+	    else if(!g_strcasecmp(arg, "--open") ||
+		    !g_strcasecmp(arg, "-open") ||
+		    !strcmp(arg, "--o") ||
+		    !strcmp(arg, "-o")
 	    )
 	    {
 		open_object = TRUE;
 	    }
-	    /* (Re)download last object? */
-	    else if(!g_strcasecmp(arg_ptr, "--last") ||
-		    !g_strcasecmp(arg_ptr, "-last") ||
-		    !g_strcasecmp(arg_ptr, "--l") ||
-		    !g_strcasecmp(arg_ptr, "-l")
+	    /* (Re)download the last object? */
+	    else if(!g_strcasecmp(arg, "--last") ||
+		    !g_strcasecmp(arg, "-last") ||
+		    !strcmp(arg, "--l") ||
+		    !strcmp(arg, "-l")
 	    )
 	    {
 		download_last_object = TRUE;
 	    }
+	    /* Check download progress and update messages interval
+	     * in milliseconds (default is 1000)?
+	     */
+	    else if(!g_strcasecmp(arg, "--interval") ||
+		    !g_strcasecmp(arg, "-interval") ||
+		    !g_strcasecmp(arg, "--int") ||
+		    !g_strcasecmp(arg, "-int") ||
+		    !g_strcasecmp(arg, "--i") ||
+		    !g_strcasecmp(arg, "-i")
+	    )
+	    {
+		i++;
+		if(i < argc)
+		{
+		    check_interval_ms = MAX(ATOL(argv[i]), 1l);
+		}
+		else
+		{
+		    g_printerr(
+"%s: Requires argument.\n",
+			arg
+		    );
+		    status = 2;
+		    DO_SHUTDOWN
+		    return(status);
+		}
+	    }
+	    /* Geometry */
+	    else if(!g_strcasecmp(arg, "--geometry") ||
+		    !g_strcasecmp(arg, "-geometry")
+	    )
+	    {
+		i++;
+		arg = (i < argc) ? argv[i] : NULL;
+		if(arg != NULL)
+		{
+		    if(geometry == NULL)
+		    {
+			gint x, y, width, height;
 
-	    /* All else assume source url or destination path */
+			geometry = (GdkRectangle *)g_malloc0(
+			    sizeof(GdkRectangle)
+			);
+			geometry_flags = GUIParseGeometry(
+			    arg, &x, &y, &width, &height
+			);
+			geometry->x = x;
+			geometry->y = y;
+			geometry->width = width;
+			geometry->height = height;
+		    }
+		}
+		else 
+		{
+		    g_printerr(
+			"%s: Requires argument\n",
+			argv[i - 1]
+		    );
+		    status = 2;
+		    DO_SHUTDOWN
+		    return(status);
+		}
+	    }
+	    /* Single character argument? */
+	    else if((*arg == '-') ? (arg[1] != '-') : FALSE)
+	    {
+		const gchar *v = arg + 1;
+		gchar c;
+
+		while(*v != '\0')
+		{
+		    c = *v;
+		    if(c == 'b')
+		    {
+			beep_when_complete = TRUE;
+		    }
+		    else if(c == 'c')
+		    {
+			 need_confirmation = TRUE;
+		    }
+		    else if(c == 'o')
+		    {
+			 open_object = TRUE;
+		    }
+		    else if(c == 'l')
+		    {
+			 download_last_object = TRUE;
+		    }
+		    else
+		    {
+			g_printerr(
+"-%c: Unsupported argument.\n",
+			    c
+			);
+			status = 2;
+			DO_SHUTDOWN
+			return(status);
+		    }
+		    v++;
+		}
+	    }
+	    /* Non-option argument? */
+	    else if((*arg != '-') && (*arg != '+'))
+	    {
+		/* All else assume source url or destination path */
+		if(source_url == NULL)
+		    source_url = STRDUP(arg);
+		else if(destination_path == NULL)
+		    destination_path = STRDUP(arg);
+	    }
 	    else
 	    {
-		if(source_url == NULL)
-		    source_url = STRDUP(arg_ptr);
-		else if(destination_path == NULL)
-		    destination_path = STRDUP(arg_ptr);
+		g_printerr(
+"%s: Unsupported argument.\n",
+		    arg
+		);
+		status = 2;
+		DO_SHUTDOWN
+		return(status);
 	    }
 	}
 
@@ -1437,15 +1664,10 @@ to (re)download.",
 	/* Use current working directory as destination path if the
 	 * destination path is not obtained from the command line.
 	 */
-	if((destination_path != NULL) ? (*destination_path == '\0') : TRUE)
+	if(STRISEMPTY(destination_path))
 	{
-	    gchar cwd[PATH_MAX];
-	    if(getcwd(cwd, PATH_MAX) != NULL)
-	    {
-		cwd[PATH_MAX - 1] = '\0';
-		g_free(destination_path);
-		destination_path = STRDUP(cwd);
-	    }
+	    g_free(destination_path);
+	    destination_path = STRDUP(g_get_current_dir());
 	}
 
 
@@ -1509,6 +1731,7 @@ to (re)download.",
 		":", destination_path,
 		NULL, DownloadBrowseTargetPathCB
 	    );
+	    PDialogSetPromptCompletePath(-1);
 	    PDialogSetPromptTip(
 		-1,
 #if defined(PROG_LANGUAGE_SPANISH)
@@ -1542,6 +1765,20 @@ to (re)download.",
  object to or leave it blank to download to the current directory"
 #endif
 	    );
+	    if(geometry != NULL)
+	    {
+		GtkWidget *w = PDialogGetToplevel();
+		if(w != NULL)
+		{
+		    gtk_widget_set_uposition(
+			w,
+			(geometry_flags & GDK_GEOMETRY_X) ?
+			    geometry->x : 0,
+			(geometry_flags & GDK_GEOMETRY_Y) ?
+			    geometry->y : 0
+		    );
+		}
+	    }
 	    PDialogSetSize(450, -1);
 	    strv = PDialogGetResponseIconData(
 		PROG_NAME,
@@ -1651,13 +1888,8 @@ object will be downloaded to the current directory."
 	 */
 	if(STRISEMPTY(destination_path))
 	{
-	    gchar cwd[PATH_MAX];
-	    if(getcwd(cwd, PATH_MAX) != NULL)
-	    {
-		cwd[PATH_MAX - 1] = '\0';
-		g_free(destination_path);
-		destination_path = STRDUP(cwd);
-	    }
+	    g_free(destination_path);
+	    destination_path = STRDUP(g_get_current_dir());
 	}
 
 
@@ -1674,7 +1906,7 @@ object will be downloaded to the current directory."
 	    else
 		destination_file = STRDUP(destination_path);
 
-	    chdir(destination_path);
+	    EDVSetCWD(destination_path);
 	}
 	else if(destination_path != NULL)
 	{
@@ -1684,8 +1916,7 @@ object will be downloaded to the current directory."
 	     */
 	    const gchar	*s = strrchr(source_url, '/'),
 			*parent = GetParentDir(destination_path);
-	    if(parent != NULL)
-		chdir(parent);
+	    EDVSetCWD(parent);
 
 	    if(s != NULL)
 		destination_file = g_strdup_printf(
@@ -1696,13 +1927,8 @@ object will be downloaded to the current directory."
 	}
 	else
 	{
-	    gchar cwd[PATH_MAX];
-	    if(getcwd(cwd, PATH_MAX) != NULL)
-	    {
-		cwd[PATH_MAX - 1] = '\0';
-		if(destination_path == NULL)
-		    destination_path = STRDUP(cwd);
-	    }
+	    g_free(destination_path);
+	    destination_path = STRDUP(g_get_current_dir());
 	}
 
 
@@ -2144,9 +2370,10 @@ objects.\n",
 	stdout_file = EDVTmpName(NULL);
 
 
-	/* Execute download program and monitor download */
+	/* Execute the download program and monitor the download */
 	if(destination_file != NULL)
 	{
+	    const gulong start_time = (gulong)time(NULL);
 	    gchar *cmd = g_strdup_printf(
 		"\"%s\" \"%s\" --tries=1 --progress=dot -c",
 		prog, source_url
@@ -2158,8 +2385,8 @@ objects.\n",
 	    const gint p = ExecOE(cmd, NULL, stdout_file);
 	    if(p <= 0)
 	    {
-		/* Unable to execute download command */
-		gchar *buf = g_strdup_printf(
+		/* An error occured while executing the download command */
+		gchar *s = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Incapaz de ejecutar la orden de la descarga:\n\
 \n\
@@ -2189,7 +2416,7 @@ objects.\n",
 \n\
     %s\n"
 #else
-"Unable to execute download command:\n\
+"An error occured while executing the download command:\n\
 \n\
     %s\n"
 #endif
@@ -2199,51 +2426,51 @@ objects.\n",
 		CDialogGetResponse(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Descargue Fallado",
-		    buf,
+		    s,
 "La orden utiliz para descargar el objeto no se podra\n\
 ejecutar, verifica por favor que el programa de la descarga\n\
 existe en la ubicacin especificada encima de y es ejecutable.\n",
 #elif defined(PROG_LANGUAGE_FRENCH)
 "ECHEC de tlchargement",
-		    buf,
+		    s,
 "La commande utilis pour tlcharger l'objet ne pourrait pas\n\
 tre excut, s'il vous plat vrifier que le programme de\n\
 chargement existe  l'emplacement spcifi au-dessus  et est\n\
 xcutable.\n",
 #elif defined(PROG_LANGUAGE_GERMAN)
 "Laden Sie Versagt",
-		    buf,
+		    s,
 "Der befehl knnte das objekt nicht durchgefhrt werden hatte\n\
 geladen, beglaubigt bitte, da das programm ldt, am Ort ober\n\
 existiert hat angegeben und ist ausfhrbar.\n",
 #elif defined(PROG_LANGUAGE_ITALIAN)
 "Scaricare Fallito",
-		    buf,
+		    s,
 "Il comando ha usato per scaricare l'oggetto non potrebbe\n\
 essere eseguito, verifica per favore che lo scarica il programma\n\
 esiste alla posizione specificata e  sopra eseguibile.\n",
 #elif defined(PROG_LANGUAGE_DUTCH)
 "Download Geverzuimenene",
-		    buf,
+		    s,
 "Het bevel zou het voorwerp niet kunnen uitgevoerd worden heeft\n\
 gedownload, alstublieft bevestigt dat het programma downloadt\n\
 aan de plaats bovenstaande bestaat specificeerde en uitvoerbaar\n\
 is.\n",
 #elif defined(PROG_LANGUAGE_PORTUGUESE)
 "Baixe Fracassado",
-		    buf,
+		    s,
 "O comando baixava o objeto no podia ser executado, por favor\n\
 verifica-se que o programa de download existe na localidade\n\
 especificado acima e est executable.\n",
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "Download Sviktet",
-		    buf,
+		    s,
 "Kommandoen som brukt til download objektet ikke utfrer, vr\n\
 s snill og bekreft at download program finnes p plasseringen\n\
 spesifisert ovenfor og er gjennomfrbar.\n",
 #else
 "Download Failed",
-		    buf,
+		    s,
 "The command used to download the object could not be executed,\n\
 please verify that the download program exists at the location\n\
 specified above and is executable.\n",
@@ -2253,17 +2480,28 @@ specified above and is executable.\n",
 		    CDIALOG_BTNFLAG_OK
 		);
 		CDialogSetTransientFor(NULL);
-		g_free(buf);
+		g_free(s);
 		status = 1;
 	    }
 	    else
 	    {
+		gint edv_history_status = 0;
+		gulong end_time;
+
 		/* Download command executed, now monitor the download
 	         * and handle the result
 		 */
-		gint download_status = DownloadMonitor(
-		    ctx, p, source_url, destination_file, stdout_file
+		const gint download_status = DownloadMonitor(
+		    ctx, p, check_interval_ms,
+		    source_url, destination_file, stdout_file,
+		    geometry_flags,
+		    geometry
 		);
+
+		/* Get the ending time of the download */
+		end_time = (gulong)time(NULL);
+
+		/* Check the result of the download */
 		switch(download_status)
 		{
 		  case 0:	/* Success */
@@ -2279,35 +2517,50 @@ specified above and is executable.\n",
 			g_list_foreach(paths_list, (GFunc)g_free, NULL);
 			g_list_free(paths_list);
 		    }
+		    edv_history_status = 0;
 		    break;
 
 		  case -2:	/* Process exited but object not completely downloaded */
-		    status = 2;
 		    if(beep_when_complete)
 			EDVPlaySoundCompleted(ctx);
+		    status = 2;
+		    edv_history_status = -1;
 		    break;
 
 		  case -3:	/* Object not downloaded (does not exist) */
 		    status = 2;
+		    edv_history_status = -2;
 		    break;
 
 		  case -4:	/* User abort */
 		    status = 4;
+		    edv_history_status = -4;
 		    break;
 		}
 
 		/* Record the last downloaded URL regardless of the
-		 * above result.
+		 * above result
 		 */
 		DownloadCFGSetLast(
 		    ctx, source_url, destination_path, download_status
 		);
 
+		/* Append history to Endeavour */
+		EDVHistoryAppend(
+		    ctx,
+		    EDV_HISTORY_DISK_OBJECT_DOWNLOAD,
+		    start_time, end_time,
+		    edv_history_status,
+		    source_url,
+		    destination_path,
+		    NULL		/* No comments */
+		);
 	    }
-	}	/* Execute download program and monitor download */
+	}	/* Execute the download program and monitor the download */
 
 
 	DO_SHUTDOWN
+
 	return(status);
 #undef DO_SHUTDOWN
 }
