@@ -11,7 +11,7 @@
 
 #include "../guiutils.h"
 #include "../cdialog.h"
-#include "../edvtypes.h"
+#include "../edv_types.h"
 
 #include "formatmanager.h"
 #include "formatcb.h"
@@ -890,7 +890,7 @@ format stage.",
 #ifdef PROG_LANGUAGE_SPANISH
 		"El Format Fall",
 		buf,
-		NULL
+		NULL,
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
 		"Le Formattage a Echou",
