@@ -402,9 +402,9 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
 	heading[0] = "Nom";
-	heading[1] = "Sentier D'appareil";
-	heading[2] = "Monter Le Sentier";
-	heading[3] = "Classer Le Systme";
+	heading[1] = "Composant";
+	heading[2] = "Rpertoire de montage";
+	heading[3] = "Systme de fichier";
 #endif
 	fm->devices_clist = w = gtk_clist_new_with_titles(4, heading);
 	clist = GTK_CLIST(w);
@@ -476,7 +476,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 	    label = "Comienzo";
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-	    label = "Dbut";
+	    label = "Dmarrer";
 #endif
 	    accel_key = 0;
 	    accel_mods = 0;
@@ -549,7 +549,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 	    "El Formato Rpido"
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-	    "Format Rapide"
+	    "Formattage Rapide"
 #endif
 	);
 	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
@@ -562,7 +562,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 "Verifique esto saltarse la etapa de formato y slo crear sistema del archivo"
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-"Vrifier ceci pour sauter l'tape de format et cre seulement systme de fichier"
+"Sauter l'tape de formattage et cre seulement le systme de fichier"
 #endif
 	);
 	gtk_widget_show(w);
@@ -615,7 +615,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 ""
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-""
+"Choisir la capacit actuelle du disque"
 #endif
 	);
 
@@ -629,7 +629,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 	    "Etiqueta Volumen:",
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-	    "Etiquette Volume:",
+	    "Nom du Volume:",
 #endif
 	    NULL, &entry_rtn
 	);
@@ -647,7 +647,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 "Entre la etiqueta del volumen para ser colocada en los medios despus de formato"
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-"Entrer l'tiquette de volume tre place sur le presse aprs le format"
+"Entrer le nom du volume qui sera enregist sue le support"
 #endif
 	);
 
@@ -672,7 +672,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 	    "Archive El Tipo De Sistema:",
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-	    "Classer Le Type De Systme:",
+	    "Type de Systme de fichier:",
 #endif
 	    "",
 	    glist,
@@ -702,7 +702,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 "Escoja el tipo de sistema de archivo para ser creado en los medios"
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-"Choisir le type de systme de fichier tre cr sur le presse"
+"Choisir le type du systme de fichier pour ce support"
 #endif
 	);
 
@@ -716,7 +716,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 	    "Detallado"
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-	    "Verbose"
+	    "Bavard"
 #endif
 	);
 	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
@@ -729,7 +729,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 	    "Verifique este despliegue los mensajes adicionales durante formato"
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-	    "Vrifier cette exposition ces messages dtaills pendant le format"
+	    "Donner les messages dtaills pendant le formattage"
 #endif
 	);
 	gtk_widget_show(w);
@@ -783,7 +783,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 	    "Comienzo",
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-	    "Dbut",
+	    "Dmarrage",
 #endif
 	    NULL
 	);
@@ -803,7 +803,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 	    "El comienzo formatting la media el artefacto escogido"
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-	    "Le dbut pour formatting le media l'appareil choisi"
+	    "Dmarrage du programme de formattage"
 #endif
 	);
 	gtk_signal_connect(
@@ -851,7 +851,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 	    "Pare la corriente el proceso que formatting"
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-	    "Arrter le courant procd qui formatting"
+	    "Arrter la commande de formattage"
 #endif
 	);
 	gtk_signal_connect(
@@ -892,7 +892,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 	    "Resultados"
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-	    "Rsultats"
+	    "Compte-rendu"
 #endif
 	);
 	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
@@ -977,7 +977,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 	    "Claro",
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-	    "Clair",
+	    "Effacer",
 #endif
 	    NULL
 	);
@@ -997,7 +997,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 	    "Los resultados claros"
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-	    "Rsultats clairs"
+	    "Effacer le compte-rendu"
 #endif
 	);
 	gtk_signal_connect(
@@ -1027,7 +1027,7 @@ format_manager_struct *FormatManagerNew(edv_context_struct *ctx)
 	    "Cierre",
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-	    "Proche",
+	    "Quitter",
 #endif
 	    NULL
 	);
