@@ -7,8 +7,8 @@
 
 
 /* Last error message pointer */
-static const char *imgio_last_load_error;
-static const char *imgio_last_write_error;
+static const char *imgio_last_open_error;
+static const char *imgio_last_save_error;
 
 
 /*
@@ -255,12 +255,12 @@ int ImgGIFReadRGBA(
 	GifRecordType rt;
 
 	/* Reset last load error */
-	imgio_last_load_error = NULL;
+	imgio_last_open_error = NULL;
 
 	/* Skip if user aborted */
 	if(*user_aborted)
 	{
-	    imgio_last_load_error = "User aborted operation";
+	    imgio_last_open_error = "User aborted operation";
 	    return(-4);
 	}
 
@@ -268,7 +268,7 @@ int ImgGIFReadRGBA(
 	ft = DGifOpenFileName(filename);
 	if(ft == NULL)
 	{
-	    imgio_last_load_error = "Unable to open the file for reading";
+	    imgio_last_open_error = "Unable to open the file for reading";
 	    return(-1);
 	}
 
@@ -306,7 +306,7 @@ printf(
 	if(scr_buf == NULL)
 	{
 	    DGifCloseFile(ft);
-	    imgio_last_load_error = "Memory allocation error";
+	    imgio_last_open_error = "Memory allocation error";
 	    return(-3);
 	}
 
@@ -709,7 +709,7 @@ printf(
 
 	if(*user_aborted)
 	{
-	    imgio_last_load_error = "User aborted operation";
+	    imgio_last_open_error = "User aborted operation";
 	    return(-4);
 	}
 	else
@@ -820,7 +820,7 @@ static int ImgGIFDitherRGBA(
 	{
 	    free(cidx);
 	    free(colors_list);
-	    imgio_last_write_error = "User aborted operation";
+	    imgio_last_save_error = "User aborted operation";
 	    return(-4);
 	}
 
@@ -829,7 +829,7 @@ static int ImgGIFDitherRGBA(
 	if(data == NULL)
 	{
 	    free(colors_list);
-	    imgio_last_write_error =
+	    imgio_last_save_error =
 		"Unable to generate color index image";
 	    return(-1);
 	}
@@ -838,7 +838,7 @@ static int ImgGIFDitherRGBA(
 	if(colors_list == NULL)
 	{
 	    free(data);
-	    imgio_last_write_error =
+	    imgio_last_save_error =
 		"Unable to generate color palette";
 	    return(-1);
 	}
@@ -855,7 +855,7 @@ static int ImgGIFDitherRGBA(
 	    if(colors_list == NULL)
 	    {
 		free(data);
-		imgio_last_write_error = "Memory allocation error";
+		imgio_last_save_error = "Memory allocation error";
 		return(-3);
 	    }
 	    c = &colors_list[i];
@@ -885,7 +885,7 @@ static int ImgGIFDitherRGBA(
 	    );
 	    if(colors_list == NULL)
 	    {
-		imgio_last_write_error = "Memory allocation error";
+		imgio_last_save_error = "Memory allocation error";
 		return(-3);
 	    }
 	    c = &colors_list[i];
@@ -940,7 +940,7 @@ static int ImgGIFDitherRGBA(
 	{
 	    free(data);
 	    free(colors_list);
-	    imgio_last_write_error = "Memory allocation error";
+	    imgio_last_save_error = "Memory allocation error";
 	    return(-3);
 	}
 
@@ -1004,7 +1004,7 @@ static int ImgWriteGIFFileRGBAToColor(
 	ft = EGifOpenFileName(filename, 0);
 	if(ft == NULL)
 	{
-	    imgio_last_write_error = "Unable to open the file for writing";
+	    imgio_last_save_error = "Unable to open the file for writing";
 	    return(-1);
 	}
 
@@ -1198,7 +1198,7 @@ static int ImgWriteGIFFileRGBAToColor(
 
 	if(*user_aborted)
 	{
-	    imgio_last_write_error = "User aborted operation";
+	    imgio_last_save_error = "User aborted operation";
 	    return(-4);
 	}
 	else
@@ -1291,7 +1291,7 @@ static int ImgWriteGIFFileRGBAToGreyscale(
 	if(cidx == NULL)
 	{
 	    FreeMapObject(colormap);
-	    imgio_last_write_error = "Memory allocation error";
+	    imgio_last_save_error = "Memory allocation error";
 	    return(-3);
 	}
 
@@ -1301,7 +1301,7 @@ static int ImgWriteGIFFileRGBAToGreyscale(
 	{
 	    FreeMapObject(colormap);
 	    free(cidx);
-	    imgio_last_write_error = "Unable to open the file for writing";
+	    imgio_last_save_error = "Unable to open the file for writing";
 	    return(-1);
 	}
 	 
@@ -1499,7 +1499,7 @@ static int ImgWriteGIFFileRGBAToGreyscale(
 
 	if(*user_aborted)
 	{
-	    imgio_last_write_error = "User aborted operation";
+	    imgio_last_save_error = "User aborted operation";
 	    return(-4);
 	}
 	else
@@ -1623,7 +1623,7 @@ static int ImgWriteGIFFileRGBAToBW(
 	if(cidx == NULL)
 	{
 	    FreeMapObject(colormap);
-	    imgio_last_write_error = "Memory allocation error";
+	    imgio_last_save_error = "Memory allocation error";
 	    return(-3);
 	}
 
@@ -1633,7 +1633,7 @@ static int ImgWriteGIFFileRGBAToBW(
 	{
 	    FreeMapObject(colormap);
 	    free(cidx);
-	    imgio_last_write_error = "Unable to open the file for writing";
+	    imgio_last_save_error = "Unable to open the file for writing";
 	    return(-1);
 	}
 
@@ -1832,7 +1832,7 @@ static int ImgWriteGIFFileRGBAToBW(
 
 	if(*user_aborted)
 	{
-	    imgio_last_write_error = "User aborted operation";
+	    imgio_last_save_error = "User aborted operation";
 	    return(-4);
 	}
 	else
@@ -1877,14 +1877,14 @@ int ImgWriteGIFFileRGBA(
 	int user_aborted = 0;
 
 	/* Reset global last write error message pointer */
-	imgio_last_write_error = NULL;
+	imgio_last_save_error = NULL;
 
 	if(STRISEMPTY(filename) || (rgba == NULL) || (nframes <= 0) ||
 	   (width <= 0) || (height <= 0) ||
 	   (bpp != 4)
 	)
 	{
-	    imgio_last_write_error = "Invalid value used to describe image";
+	    imgio_last_save_error = "Invalid value used to describe image";
 	    return(-2);
 	}
 
@@ -1948,7 +1948,7 @@ int ImgWriteGIFFileRGBA(
 	    break;
 
 	  default:
-	    imgio_last_write_error = "Unsupported GIF color format";
+	    imgio_last_save_error = "Unsupported GIF color format";
 	    status = -2;
 	    break;
 	}
@@ -1968,7 +1968,7 @@ int ImgWriteGIFFileRGBA(
 
 	if(user_aborted)
 	{
-	    imgio_last_write_error = "User aborted operation";
+	    imgio_last_save_error = "User aborted operation";
 	    return(-4);
 	}
 	else
