@@ -1,9 +1,8 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkprivate.h>
-#if defined(WIN32)
+#if defined(_WIN32)
 # include <gdk/gdkwin32.h>
-# define HAVE_WIN32
 #else
 # include <gdk/gdkx.h>
 # define HAVE_X
