@@ -271,38 +271,42 @@ gint HViewExposeCB(
 	/* Draw text */
 	if((buf != NULL) && (hv->font != NULL))
 	{
-	    gint border = 0;
+	    gboolean is_selected;
+	    const gint	border = 0,
+			edit_buf_pos = hv->edit_buf_i;
+	    const gchar *edit_buf = hv->edit_buf;
 	    gint i, j, n, x, y;
 	    GdkFont *font = hv->font;
 	    gint buf_start = (gint)adj->value / cell_height * cells_per_row;
 	    gint y_start = border - ((gint)adj->value % cell_height);
-	    gint sel_start = hv->buf_sel_start;
-	    gint sel_end = hv->buf_sel_end;
+	    gint	sel_start = hv->buf_sel_start,
+			sel_end = hv->buf_sel_end;
 	    gchar tmp_buf[80];
 
-
 	    /* Set up gc for drawing text */
 	    gdk_gc_set_font(gc, font);
 
-	    /* Sanitize selection bounds */
+	    /* Sanitize the selection bounds */
 	    if((sel_start > -1) && (sel_end > -1))
 	    {
 		if(sel_end < sel_start)
 		{
-		    gint t = sel_start;
+		    const gint t = sel_start;
 		    sel_start = sel_end;
 		    sel_end = t;
 		}
 	    }
 
-	    /* Set starting positions */
-	    i = buf_start;
+	    /* Set the starting positions */
+	    i = buf_start;		/* Buffer index */
 	    if(i < 0)
 		i = 0;
-	    y = y_start;
+	    y = y_start;		/* Line */
+
+	    /* Draw each line */
 	    while((i < buf_len) && (y < height))
 	    {
-		/* Draw address */
+		/* Draw the address */
 		x = hv->address_x;
 		g_snprintf(
 		    tmp_buf, sizeof(tmp_buf),
@@ -314,7 +318,12 @@ gint HViewExposeCB(
 		    x, y + font->ascent, tmp_buf
 		);
 
-		/* Draw hex values */
+		/* Draw each hex value
+		 *
+		 * n is the number of hex values drawn for this line
+		 * j is the buffer index
+		 * x is the x coordinate position in window coordinates
+		 */
 		for(n = 0, j = i, x = hv->hex_x;
 		    n < cells_per_row;
 		    n++, j++, x += (3 * cell_width)
@@ -323,12 +332,10 @@ gint HViewExposeCB(
 		    if(j >= buf_len)
 			break;
 
-		    /* Draw selected color background if this cell falls
-		     * within the selection bounds?  Also set text color
-		     * for upcomming string drawing.
-		     */
+		    /* Is this hex value selected? */
 		    if((j >= sel_start) && (j <= sel_end))
 		    {
+			/* Draw the selected background */
 			gdk_draw_rectangle(
 			    drawable,
 			    style->bg_gc[GTK_STATE_SELECTED],
@@ -338,23 +345,52 @@ gint HViewExposeCB(
 			gdk_gc_set_foreground(
 			    gc, &style->text[GTK_STATE_SELECTED]
 			);
+			is_selected = TRUE;
 		    }
 		    else
 		    {
 			gdk_gc_set_foreground(gc, &style->text[state]);
+			is_selected = FALSE;
 		    }
-		    g_snprintf(
-			tmp_buf, sizeof(tmp_buf),
-			"%.2X", buf[j]
-		    );
-		    gdk_draw_string(
-			drawable, font, gc,
-			x, y + font->ascent, tmp_buf
-		    );
 
-		    /* Cursor rectangle */
+		    /* Is a value being edited? */
+		    if((edit_buf_pos > 0) ?
+			(is_selected ? TRUE : (j == hv->buf_pos)) : FALSE
+		    )
+		    {
+			/* Draw the hex value from the edit buffer */
+			const gint m = MIN(edit_buf_pos, sizeof(hv->edit_buf));
+			gint i;
+
+			for(i = 0; i < m; i++)
+			    tmp_buf[i] = edit_buf[i];
+			tmp_buf[i] = '_';
+			tmp_buf[i + 1] = '\0';
+
+			gdk_draw_string(
+			    drawable, font, gc,
+			    x, y + font->ascent,
+			    tmp_buf
+			);
+		    }
+		    else
+		    {
+			/* Draw the hex value */
+			g_snprintf(
+			    tmp_buf, sizeof(tmp_buf),
+			    "%.2X", buf[j]
+			);
+			gdk_draw_string(
+			    drawable, font, gc,
+			    x, y + font->ascent,
+			    tmp_buf
+			);
+		    }
+
+		    /* Is the cursor over this value? */
 		    if(j == hv->buf_pos)
 		    {
+			/* Draw the cursor */
 			GdkGCValues gcv;
 			gdk_gc_get_values(
 			    style->text_gc[GTK_STATE_SELECTED], &gcv
@@ -375,7 +411,12 @@ gint HViewExposeCB(
 		    }
 		}
 
-		/* Draw ascii values */
+		/* Draw each ASCII value
+		 *
+		 * n is the number of hex values drawn for this line
+		 * j is the buffer index
+		 * x is the x coordinate position in window coordinates
+		 */
 		for(n = 0, j = i, x = hv->ascii_x;
 		    n < cells_per_row;
 		    n++, j++, x += cell_width
@@ -483,13 +524,14 @@ gint HViewKeyCB(
 	  case GDK_Escape:
 	    if(key_press)
 	    {
-		/* Cancel user editing of current hex value (if the user
-		 * was doing that)
-		 */
-		hv->buf_hex_edit = 0;
-		*hv->buf_hex_edit_buf = '\0';
-		HViewQueueDraw(hv);
-		HViewSetStatusMessage(hv, NULL, FALSE);
+		/* Cancel user editing of current hex value */
+		if(hv->edit_buf_i > 0)
+		{
+		    hv->edit_buf_i = 0;
+		    *hv->edit_buf = '\0';
+		    HViewQueueDraw(hv);
+		    HViewSetStatusMessage(hv, NULL, FALSE);
+		}
 	    }
 	    status = TRUE;
 	    break;
@@ -497,10 +539,11 @@ gint HViewKeyCB(
 	  case GDK_BackSpace:
 	    if(key_press)
 	    {
-		if(hv->buf_hex_edit > 0)
-		    hv->buf_hex_edit--;
-		if(hv->buf_hex_edit < 3)
-		    hv->buf_hex_edit_buf[hv->buf_hex_edit] = '\0';
+		if(hv->edit_buf_i > 0)
+		    hv->edit_buf_i--;
+		if(hv->edit_buf_i < sizeof(hv->edit_buf))
+		    hv->edit_buf[hv->edit_buf_i] = '\0';
+		HViewQueueDraw(hv);
 		HViewSetStatusMessage(hv, NULL, FALSE);
 	    }
 	    status = TRUE;
@@ -509,6 +552,7 @@ gint HViewKeyCB(
 	  case GDK_space:
 	    if(key_press)
 	    {
+		/* Select region? */
 		if((key->state & GDK_CONTROL_MASK) ||
 		   (key->state & GDK_SHIFT_MASK)
 		)
@@ -517,6 +561,7 @@ gint HViewKeyCB(
 		}
 		else
 		{
+		    /* Toggle select */
 		    if(HVIEW_IS_BUF_SELECTED(hv))
 			hv->buf_sel_start = hv->buf_sel_end = -1;
 		    else
@@ -525,6 +570,12 @@ gint HViewKeyCB(
 		HViewQueueDraw(hv);
 		HViewUpdate(hv);
 		HViewSetStatusMessage(hv, NULL, FALSE);
+		if(hv->select_cb != NULL)
+		    hv->select_cb(
+			hv,
+			hv->buf_sel_start, hv->buf_sel_end,
+			hv->select_data
+		    );
 	    }
 	    status = TRUE;
 	    break;
@@ -545,19 +596,19 @@ gint HViewKeyCB(
 		     * value, then finish it by assuming the second
 		     * character is a '0'
 		     */
-		    if(hv->buf_hex_edit == 1)
+		    if((hv->edit_buf_i == 1) && (sizeof(hv->edit_buf) >= 3))
 		    {
-			hv->buf_hex_edit_buf[1] = '0';
-			hv->buf_hex_edit_buf[2] = '\0';
+			hv->edit_buf[1] = '0';
+			hv->edit_buf[2] = '\0';
 		    }
 		    /* Process new value only if one or more characters
 		     * in hex edit buffer, this is so that if this key
 		     * is pressed while nothing is edit then nothing is
 		     * edited
 		     */
-		    if(hv->buf_hex_edit > 0)
+		    if(hv->edit_buf_i > 0)
 		    {
-			v = HViewHexConvert(hv->buf_hex_edit_buf);
+			v = HViewHexConvert(hv->edit_buf);
 
 			/* If selected, then set selection.  Otherwise
 			 * just set the cursor position.
@@ -569,8 +620,8 @@ gint HViewKeyCB(
 		    }
 
 		    /* Clear hex edit buffer */
-		    hv->buf_hex_edit = 0;
-		    *hv->buf_hex_edit_buf = '\0';
+		    hv->edit_buf_i = 0;
+		    *hv->edit_buf = '\0';
 
 		    hv->modified = changed = TRUE;
 		}
@@ -607,6 +658,13 @@ gint HViewKeyCB(
 
 		if(changed && (hv->changed_cb != NULL))
 		    hv->changed_cb(hv, hv->changed_data);
+
+		if(hv->select_cb != NULL)
+		    hv->select_cb(
+			hv,
+			hv->buf_sel_start, hv->buf_sel_end,
+			hv->select_data
+		    );
 	    }
 	    status = TRUE;
 	    break;
@@ -641,6 +699,13 @@ gint HViewKeyCB(
 	  case GDK_Up:
 	    if(key_press)
 	    {
+		/* Cancel editing of current hex value */
+		if(hv->edit_buf_i > 0)
+		{
+		    hv->edit_buf_i = 0;
+		    *hv->edit_buf = '\0';
+		}
+
 	        if((key->state & GDK_CONTROL_MASK) &&
 	           (adj != NULL)
 		)
@@ -661,13 +726,21 @@ gint HViewKeyCB(
 		    if(key->state & GDK_SHIFT_MASK)
 		    {
 			if(HVIEW_IS_BUF_SELECTED(hv))
+			{
 			    hv->buf_sel_end = hv->buf_pos;
+			}
 			else
 			{
 			    hv->buf_sel_start = last_pos;
 			    hv->buf_sel_end = hv->buf_pos;
 			}
 			HViewUpdate(hv);
+			if(hv->select_cb != NULL)
+			    hv->select_cb(
+				hv,
+				hv->buf_sel_start, hv->buf_sel_end,
+				hv->select_data
+			    );
 		    }
 		    HViewQueueDraw(hv);
 		    HViewSetStatusMessage(hv, NULL, FALSE);
@@ -684,6 +757,13 @@ gint HViewKeyCB(
 	  case GDK_Down:
 	    if(key_press)
 	    {
+		/* Cancel editing of current hex value */
+		if(hv->edit_buf_i > 0)
+		{
+		    hv->edit_buf_i = 0;
+		    *hv->edit_buf = '\0';
+		}
+
 		if((key->state & GDK_CONTROL_MASK) &&
 		   (adj != NULL)
 		)
@@ -711,6 +791,12 @@ gint HViewKeyCB(
 			    hv->buf_sel_end = hv->buf_pos;
 			}
 			HViewUpdate(hv);
+			if(hv->select_cb != NULL)
+			    hv->select_cb(
+				hv,
+				hv->buf_sel_start, hv->buf_sel_end,
+				hv->select_data
+			    );
 		    }
 		    HViewQueueDraw(hv);
 		    HViewSetStatusMessage(hv, NULL, FALSE);
@@ -727,6 +813,13 @@ gint HViewKeyCB(
 	  case GDK_Left:
 	    if(key_press)
 	    {
+		/* Cancel editing of current hex value */
+		if(hv->edit_buf_i > 0)
+		{
+		    hv->edit_buf_i = 0;
+		    *hv->edit_buf = '\0';
+		}
+
 	        if((key->state & GDK_CONTROL_MASK) &&
 	           (adj != NULL)
 		)
@@ -749,6 +842,12 @@ gint HViewKeyCB(
 			    hv->buf_sel_end = hv->buf_pos;
 			}
 			HViewUpdate(hv);
+			if(hv->select_cb != NULL)
+			    hv->select_cb(
+				hv,
+				hv->buf_sel_start, hv->buf_sel_end,
+				hv->select_data
+			    );
 		    }
 		    HViewQueueDraw(hv);
 		    HViewSetStatusMessage(hv, NULL, FALSE);
@@ -765,6 +864,13 @@ gint HViewKeyCB(
 	  case GDK_Right:
 	    if(key_press)
 	    {
+		/* Cancel editing of current hex value */
+		if(hv->edit_buf_i > 0)
+		{
+		    hv->edit_buf_i = 0;
+		    *hv->edit_buf = '\0';
+		}
+
 		if((key->state & GDK_CONTROL_MASK) &&
 		   (adj != NULL)
 		)
@@ -787,6 +893,12 @@ gint HViewKeyCB(
 			    hv->buf_sel_end = hv->buf_pos;
 			}
 			HViewUpdate(hv);
+			if(hv->select_cb != NULL)
+			    hv->select_cb(
+				hv,
+				hv->buf_sel_start, hv->buf_sel_end,
+				hv->select_data
+			    );
 		    }
 		    HViewQueueDraw(hv);
 		    HViewSetStatusMessage(hv, NULL, FALSE);
@@ -804,6 +916,13 @@ gint HViewKeyCB(
 	  case GDK_KP_Page_Up:
 	    if(key_press)
 	    {
+		/* Cancel editing of current hex value */
+		if(hv->edit_buf_i > 0)
+		{
+		    hv->edit_buf_i = 0;
+		    *hv->edit_buf = '\0';
+		}
+
 		if((key->state & GDK_CONTROL_MASK) && (adj != NULL))
 		{
 		    gfloat v = adj->value - adj->page_increment;
@@ -830,6 +949,12 @@ gint HViewKeyCB(
 			    hv->buf_sel_end = hv->buf_pos;
 			}
 			HViewUpdate(hv);
+			if(hv->select_cb != NULL)
+			    hv->select_cb(
+				hv,
+				hv->buf_sel_start, hv->buf_sel_end,
+				hv->select_data
+			    );
 		    }
 		    HViewQueueDraw(hv);
 		    HViewSetStatusMessage(hv, NULL, FALSE);
@@ -848,6 +973,13 @@ gint HViewKeyCB(
 	  case GDK_KP_Page_Down:
 	    if(key_press)
 	    {
+		/* Cancel editing of current hex value */
+		if(hv->edit_buf_i > 0)
+		{
+		    hv->edit_buf_i = 0;
+		    *hv->edit_buf = '\0';
+		}
+
 		if((key->state & GDK_CONTROL_MASK) && (adj != NULL))
 		{
 		    gfloat v = adj->value + adj->page_increment;
@@ -875,6 +1007,12 @@ gint HViewKeyCB(
 			    hv->buf_sel_end = hv->buf_pos;
 			}
 			HViewUpdate(hv);
+			if(hv->select_cb != NULL)
+			    hv->select_cb(
+				hv,
+				hv->buf_sel_start, hv->buf_sel_end,
+				hv->select_data
+			    );
 		    }
 		    HViewQueueDraw(hv);
 		    HViewSetStatusMessage(hv, NULL, FALSE);
@@ -893,6 +1031,13 @@ gint HViewKeyCB(
 	  case GDK_KP_Home:
 	    if(key_press)
 	    {
+		/* Cancel editing of current hex value */
+		if(hv->edit_buf_i > 0)
+		{
+		    hv->edit_buf_i = 0;
+		    *hv->edit_buf = '\0';
+		}
+
 		if((key->state & GDK_CONTROL_MASK) && (adj != NULL))
 		{
 		    gfloat v = adj->lower;
@@ -906,13 +1051,21 @@ gint HViewKeyCB(
 		    if(key->state & GDK_SHIFT_MASK)
 		    {
 			if(HVIEW_IS_BUF_SELECTED(hv))
+			{
 			    hv->buf_sel_end = hv->buf_pos;
+			}
 			else
 			{   
 			    hv->buf_sel_start = last_pos;
 			    hv->buf_sel_end = hv->buf_pos;
 			}
 			HViewUpdate(hv);
+			if(hv->select_cb != NULL)
+			    hv->select_cb(
+				hv,
+				hv->buf_sel_start, hv->buf_sel_end,
+				hv->select_data
+			    );
 		    }
 		    HViewQueueDraw(hv);
 		    HViewSetStatusMessage(hv, NULL, FALSE);
@@ -931,6 +1084,13 @@ gint HViewKeyCB(
 	  case GDK_KP_End:
 	    if(key_press)
 	    {
+		/* Cancel editing of current hex value */
+		if(hv->edit_buf_i > 0)
+		{
+		    hv->edit_buf_i = 0;
+		    *hv->edit_buf = '\0';
+		}
+
 		if((key->state & GDK_CONTROL_MASK) && (adj != NULL))
 		{
 		    gfloat v = adj->upper - adj->page_size;
@@ -946,13 +1106,21 @@ gint HViewKeyCB(
 		    if(key->state & GDK_SHIFT_MASK)
 		    {
 			if(HVIEW_IS_BUF_SELECTED(hv))
+			{
 			    hv->buf_sel_end = hv->buf_pos;
+			}
 			else
 			{   
 			    hv->buf_sel_start = last_pos;
 			    hv->buf_sel_end = hv->buf_pos;
 			}
 			HViewUpdate(hv);
+			if(hv->select_cb != NULL)
+			    hv->select_cb(
+				hv,
+				hv->buf_sel_start, hv->buf_sel_end,
+				hv->select_data
+			    );
 		    }
 		    HViewQueueDraw(hv);
 		    HViewSetStatusMessage(hv, NULL, FALSE);
@@ -968,9 +1136,11 @@ gint HViewKeyCB(
 	    break;
 
 	  default:
-	    /* Hex edit mode? check if number or alphabet character */
-	    if(isxdigit(key->keyval) && !hv->read_only &&
-	       (hv->edit_mode == HVIEW_EDIT_MODE_HEX) &&
+	    /* Hex edit mode? */
+	    if((hv->edit_mode == HVIEW_EDIT_MODE_HEX) &&
+	       !hv->read_only &&
+	       isxdigit((int)key->keyval) &&
+	       !(0xFFFFFF00 & key->keyval) &&
 	       ((key->state == 0x00000000) ||
 		(key->state == GDK_SHIFT_MASK)
 	       )
@@ -978,28 +1148,91 @@ gint HViewKeyCB(
 	    {
 		if(key_press)
 		{
-		    gint i = hv->buf_hex_edit;
+		    const gint i = hv->edit_buf_i;
 
 		    /* Add new character to the tempory hex edit
 		     * buffer and increment hex edit buffer position.
 		     * Do this only if the current hex edit buffer
 		     * position is in bounds.
 		     */
-		    if((i >= 0) && (i < 2))
+		    if((i >= 0) && (i <= 1) &&
+		       (sizeof(hv->edit_buf) >= (i + 1))
+		    )
 		    {
-			hv->buf_hex_edit_buf[i] =
+			hv->edit_buf[i] =
 			    toupper((gchar)key->keyval);
-			hv->buf_hex_edit_buf[i + 1] = '\0';
-			hv->buf_hex_edit++;
+			hv->edit_buf[i + 1] = '\0';
+			hv->edit_buf_i++;
 		    }
 
+		    /* Entered two characters for the hex value? */
+		    if(hv->edit_buf_i >= 2)
+		    {
+			const guint8 v = HViewHexConvert(hv->edit_buf);
+
+			/* If selected then set all values in the
+			 * selection, otherwise just set the value
+			 * at the cursor position
+			 */
+			if(HVIEW_IS_BUF_SELECTED(hv))
+			    HViewSetSelected(hv, v);
+			else if(HVIEW_IS_BUF_POS_VALID(hv, hv->buf_pos))
+			    hv->buf[hv->buf_pos] = v;
+
+			/* Clear hex edit buffer */
+			hv->edit_buf_i = 0;
+			*hv->edit_buf = '\0';
+
+			hv->modified = TRUE;
+
+			/* Seek the cursor past selection and update
+			 * the selection to the cursor's position
+			 */
+			if(hv->buf_sel_end > -1)
+			{
+			    if(hv->buf_sel_end > hv->buf_sel_start)
+				hv->buf_pos = hv->buf_sel_end + 1;
+			    else
+				hv->buf_pos = hv->buf_sel_start + 1;
+			}
+			else
+			{
+			    hv->buf_pos++;
+			}
+			if(hv->buf_pos >= hv->buf_len)
+			    hv->buf_pos = hv->buf_len - 1;
+			if(hv->buf_pos < 0)
+			    hv->buf_pos = 0;
+
+			/* Unselect */
+			hv->buf_sel_start = hv->buf_sel_end = -1;
+
+			HViewQueueDraw(hv);
+			HViewUpdate(hv);
+			if(HViewIsBufferPositionVisible(hv, hv->buf_pos) !=
+			    GTK_VISIBILITY_FULL
+			)
+			    HViewScrollTo(hv, hv->buf_pos, 0.5f);
+
+			if(hv->changed_cb != NULL)
+			    hv->changed_cb(hv, hv->changed_data);
+
+			if(hv->select_cb != NULL)
+			    hv->select_cb(
+				hv,
+				hv->buf_sel_start, hv->buf_sel_end,
+				hv->select_data
+			    );
+		    }
+
+		    HViewQueueDraw(hv);
 		    HViewSetStatusMessage(hv, NULL, FALSE);
 		}
 		status = TRUE;
 	    }
 	    /* aSCII edit mode? */
-	    else if(isascii(key->keyval) &&
-		    (hv->edit_mode == HVIEW_EDIT_MODE_ASCII) &&
+	    else if((hv->edit_mode == HVIEW_EDIT_MODE_ASCII) &&
+		    isascii((int)key->keyval) &&
 		    ((key->state == 0x00000000) ||
 		     (key->state == GDK_SHIFT_MASK)
 		    )
@@ -1012,8 +1245,8 @@ gint HViewKeyCB(
 		    else if(HVIEW_IS_BUF_POS_VALID(hv, hv->buf_pos))
 			hv->buf[hv->buf_pos] = (guint8 )key->keyval;
 
-		    /* Seek cursor past selection and update selection
-		     * to cursor's position.
+		    /* Seek the cursor past the selection and set the
+		     * selection to the cursor's position
 		     */
 		    if(HVIEW_IS_BUF_SELECTED(hv))
 		    {
@@ -1042,10 +1275,17 @@ gint HViewKeyCB(
 		    if(HViewIsBufferPositionVisible(hv, hv->buf_pos) !=
 			GTK_VISIBILITY_FULL
 		    )
-			HViewScrollTo(hv, hv->buf_pos, 0.5);
+			HViewScrollTo(hv, hv->buf_pos, 0.5f);
 
 		    if(hv->changed_cb != NULL)
 			hv->changed_cb(hv, hv->changed_data);
+
+		    if(hv->select_cb != NULL)
+			hv->select_cb(
+			    hv,
+			    hv->buf_sel_start, hv->buf_sel_end,
+			    hv->select_data
+			);
 		}
 		status = TRUE;
 	    }
@@ -1150,14 +1390,58 @@ gint HViewButtonCB(
 	  case 1:
 	    if(button_press)
 	    {
+		const gint	x = (gint)button->x,
+				y = (gint)button->y;
 		/* Focus cell, set selection, and mark dragging */
 		hv->dragging_selection = TRUE;
 
-		hv->buf_sel_start = hv->buf_pos =
-		    HViewPositionXYToBuffer(
-		    hv, (gint)button->x, (gint)button->y
-		);
-		hv->buf_sel_end = -1;
+		/* Select range? */
+		if(button->state & GDK_SHIFT_MASK)
+		{
+		    /* Is there a selection? */
+		    if((hv->buf_sel_start > -1) &&
+		       (hv->buf_sel_end > -1)
+		    )
+		    {
+			const gint i = HViewPositionXYToBuffer(hv, x, y);
+			if(i < hv->buf_pos)
+			    hv->buf_sel_start = i;
+			else
+			    hv->buf_sel_end = i;
+			hv->buf_pos = i;
+		    }
+		    /* Is the cursor set? */
+		    else if(hv->buf_pos > -1)
+		    {
+			const gint i = HViewPositionXYToBuffer(hv, x, y);
+			if(i < hv->buf_pos)
+			{
+			    hv->buf_sel_start = i;
+			    hv->buf_sel_end = hv->buf_pos;
+			}
+			else
+			{
+			    hv->buf_sel_start = hv->buf_pos;
+			    hv->buf_sel_end = i;
+			}
+			hv->buf_pos = i;
+		    }
+		    /* All else just set the cursor position and the
+		     * start of the selection
+		     */
+		    else
+		    {
+			hv->buf_sel_start = hv->buf_pos =
+			    HViewPositionXYToBuffer(hv, x, y);
+			hv->buf_sel_end = -1;
+		    }
+		}
+		else
+		{
+		    hv->buf_sel_start = hv->buf_pos =
+			HViewPositionXYToBuffer(hv, x, y);
+		    hv->buf_sel_end = -1;
+		}
 
 		HViewQueueDraw(hv);
 		HViewSetStatusMessage(hv, NULL, FALSE);
@@ -1165,6 +1449,13 @@ gint HViewButtonCB(
 		    GTK_VISIBILITY_FULL
 		)
 		    HViewScrollTo(hv, hv->buf_pos, 0.5f);
+
+		if(hv->select_cb != NULL)
+		    hv->select_cb(
+			hv,
+			hv->buf_sel_start, hv->buf_sel_end,
+			hv->select_data
+		    );
 	    }
 	    else
 	    {
@@ -1193,6 +1484,13 @@ gint HViewButtonCB(
 		HViewQueueDraw(hv);
 		HViewUpdate(hv);
 		HViewSetStatusMessage(hv, NULL, FALSE);
+
+		if(hv->select_cb != NULL)
+		    hv->select_cb(
+			hv,
+			hv->buf_sel_start, hv->buf_sel_end,
+			hv->select_data
+		    );
 	    }
 	    status = TRUE;
 	    break;
@@ -1524,6 +1822,13 @@ void HViewDeleteCB(GtkWidget *w, gpointer data)
 
 	if(hv->changed_cb != NULL)
 	    hv->changed_cb(hv, hv->changed_data);
+
+	if(hv->select_cb != NULL)
+	    hv->select_cb(
+		hv,
+		hv->buf_sel_start, hv->buf_sel_end,
+		hv->select_data
+	    );
 }
 
 /*
@@ -1694,6 +1999,12 @@ void HViewSelectAllCB(GtkWidget *w, gpointer data)
 	HViewQueueDraw(hv);
 	HViewUpdate(hv);
 	HViewSetStatusMessage(hv, NULL, FALSE);
+	if(hv->select_cb != NULL)
+	    hv->select_cb(
+		hv,
+		hv->buf_sel_start, hv->buf_sel_end,
+		hv->select_data
+	    );
 }
 
 /*
@@ -1714,6 +2025,12 @@ void HViewUnselectAllCB(GtkWidget *w, gpointer data)
 	HViewQueueDraw(hv);
 	HViewUpdate(hv);
 	HViewSetStatusMessage(hv, NULL, FALSE);
+	if(hv->select_cb != NULL)
+	    hv->select_cb(
+		hv,
+		hv->buf_sel_start, hv->buf_sel_end,
+		hv->select_data
+	    );
 }
 
 /*
@@ -1738,14 +2055,14 @@ void HViewEditModeHexCB(GtkWidget *w, gpointer data)
  */
 void HViewEditModeASCIICB(GtkWidget *w, gpointer data)
 {
-        hview_struct *hv = HVIEW(data);
-        if(hv == NULL)
-            return;
+	hview_struct *hv = HVIEW(data);
+	if(hv == NULL)
+	    return;
 
-        if(hv->freeze_count > 0)
-            return;
+	if(hv->freeze_count > 0)
+	    return;
 
-        hv->freeze_count++;
+	hv->freeze_count++;
 	HViewSetEditMode(hv, HVIEW_EDIT_MODE_ASCII);	
-        hv->freeze_count--;
+	hv->freeze_count--;
 }
