@@ -24,8 +24,9 @@
 #include "pdialog.h"
 #include "fprompt.h"
 
-#include "endeavour.h"
-#include "edvcb.h"
+#include "edv_types.h"
+#include "endeavour2.h"
+#include "edv_cb.h"
 #include "config.h"
 
 
@@ -55,7 +56,7 @@ int main(int argc, char *argv[])
 	gint err_rtn = 0;
 	gboolean	show_stdout_dialog = TRUE,
 			show_stderr_dialog = TRUE;
-	edv_core_struct *core_ptr;
+	edv_core_struct *core;
 
 	/* Reset globals */
 #ifdef HAVE_IMLIB
@@ -136,10 +137,6 @@ int main(int argc, char *argv[])
 	    }
 	}
 
-
-	/* Set up time zone */
-	tzset();
-
 	/* Set up signal callbacks */
 #ifdef SIGHUP
 	signal(SIGHUP, EDVSignalCB);
@@ -177,8 +174,8 @@ int main(int argc, char *argv[])
 	/* Need to initialize GTK? */
 	if(!initialized_gtk)
 	{
-	    /* Set GTK locale */
-            gtk_set_locale();
+	    /* Set the GTK locale */
+	    gtk_set_locale();
 
 	    /* Initialize GTK */
 	    if(!gtk_init_check(&argc, &argv))
@@ -187,7 +184,7 @@ int main(int argc, char *argv[])
 		return(1);
 	    }
 
-	    /* Initialize GDK RGB Buffers */
+	    /* Initialize the GDK RGB Buffers */
 	    gdk_rgb_init();
 
 	    initialized_gtk = TRUE;
@@ -198,7 +195,7 @@ int main(int argc, char *argv[])
 	imlib_handle = Imlib_init(GDK_DISPLAY());
 #endif  /* HAVE_IMLIB */
 
-	/* Initialize Dialogs */
+	/* Initialize the dialogs */
 	ConMsgInit(PROG_NAME_FULL, 0, 0, show_stdout_dialog, show_stderr_dialog);
 	CDialogInit();
 	FileBrowserInit();
@@ -208,18 +205,18 @@ int main(int argc, char *argv[])
 	PDialogInit();
 	ProgressDialogInit();
 
-	/* Create Core */
-	edv_core = core_ptr = EDVNew(argc, argv, &err_rtn);
-	if(core_ptr == NULL)
-	    return(err_rtn);
+	/* Create the Endeavour core */
+	edv_core = core = EDVNew(argc, argv, &err_rtn);
+	if(core == NULL)
+	    return((err_rtn != 0) ? -err_rtn : 0);
 
-	/* Enter GTK main loop */
+	/* Enter the first GTK main loop */
 	gtk_main();
 
-	/* Save the configuration and delete the Core */
+	/* Save the configuration and then delete the Endeavour core */
 	edv_core = NULL;
-	EDVDelete(core_ptr, TRUE);
-	core_ptr = NULL;
+	EDVDelete(core, TRUE);
+	core = NULL;
 
 	/* Shutdown Dialogs */
 	ProgressDialogShutdown();
