@@ -3,6 +3,8 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+#include "../include/disk.h"
+
 #include "guiutils.h"
 #include "pdialog.h"
 #include "config.h"
@@ -146,6 +148,9 @@ static gint PDialogCloseCB(
 );
 static void PDialogBrowseButtonCB(GtkWidget *widget, gpointer data);
 static void PDialogEntryEnterCB(GtkWidget *widget, gpointer data);
+static gint PDialogEntryCompletePathKeyCB(
+	GtkWidget *widget, GdkEventKey *key, gpointer data
+);
 static void PDialogButtonCB(GtkWidget *widget, gpointer data);
 static void PDialogPromptDragDataReceivedCB(
 	GtkWidget *widget, GdkDragContext *dc, gint x, gint y,
@@ -246,7 +251,8 @@ void PDialogSetPromptValue(
 	const gchar *value
 );
 void PDialogSetPromptTip(gint prompt_num, const gchar *tip);
-gchar *PDialogGetPromptValue(gint prompt_num);
+const gchar *PDialogGetPromptValue(gint prompt_num);
+void PDialogSetPromptCompletePath(gint prompt_num);
 void PDialogDeleteAllPrompts(void);
 gchar **PDialogGetResponse(
 	const gchar *title,
@@ -276,16 +282,16 @@ void PDialogUnmap(void);
 void PDialogShutdown(void);
 
 
-#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
-#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
-#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
-#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)
+#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)
 
-#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
-#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
-#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
-#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
-#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
+#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
+#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
+#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
+#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
+#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
 static pdialog_struct	pdialog;
@@ -351,7 +357,7 @@ static void PDialogBrowseButtonCB(GtkWidget *widget, gpointer data)
 }
 
 /*
- *      Dialog prompt GtkEntry "activate" signal callback.
+ *	Dialog prompt GtkEntry "activate" signal callback.
  */
 static void PDialogEntryEnterCB(GtkWidget *widget, gpointer data)
 {
@@ -365,6 +371,86 @@ static void PDialogEntryEnterCB(GtkWidget *widget, gpointer data)
 	PDialogButtonCB(d->submit_btn, d);
 }
 
+/*
+ *	Dialog prompt GtkEntry complete path "key_press_event" or
+ *	"key_release_event" signal callback.
+ */
+static gint PDialogEntryCompletePathKeyCB(
+	GtkWidget *widget, GdkEventKey *key, gpointer data
+)
+{
+	gint status = 0;
+	gboolean press;
+	GtkEntry *entry;
+	pdialog_struct *d = PDIALOG(data);
+	if((widget == NULL) || (key == NULL) || (d == NULL))
+	    return(status);
+
+	press = (key->type == GDK_KEY_PRESS) ? TRUE : FALSE;
+
+	/* Given widget must have or be a GtkEntry */
+	if(GTK_IS_COMBO(widget))
+	{
+	    GtkCombo *combo = GTK_COMBO(widget);
+	    entry = GTK_ENTRY(combo->entry);
+	}
+	else if(GTK_IS_ENTRY(widget))
+	{
+	    entry = GTK_ENTRY(widget);
+	}
+	else
+	{
+	    return(status);
+	}
+
+#define SIGNAL_EMIT_STOP	{		\
+ gtk_signal_emit_stop_by_name(			\
+  GTK_OBJECT(widget),				\
+  press ?					\
+   "key_press_event" : "key_release_event"	\
+ );						\
+}
+
+	switch(key->keyval)
+	{
+	  case GDK_Tab:
+	    /* Skip this if the shift or ctrl keys are held */
+	    if((key->state & GDK_CONTROL_MASK) ||
+	       (key->state & GDK_SHIFT_MASK)
+	    )
+		return(status);
+
+	    if(press)
+	    {
+		gchar *s = STRDUP(gtk_entry_get_text(entry));
+		if(s != NULL)
+		{
+		    gint status;
+
+		    s = CompletePath(s, &status);
+		    gtk_entry_set_text(entry, (s != NULL) ? s : "");
+		    gtk_entry_set_position(entry, -1);
+		    g_free(s);
+
+		    switch(status)
+		    {
+		      case COMPLETE_PATH_NONE:
+		      case COMPLETE_PATH_AMBIGUOUS:
+			gdk_beep();
+			break;
+		    }
+		}
+	    }
+	    SIGNAL_EMIT_STOP
+	    status = TRUE;
+	    break;
+	}
+
+#undef SIGNAL_EMIT_STOP
+
+	return(status);
+}
+
 /*
  *	Dialog GtkButton "clicked" signal callback.
  */
@@ -758,7 +844,7 @@ static void PDialogSetIcon(pdialog_struct *d, guint8 **icon_data)
 
 /*
  *	Allocates and creates a new prompt structure, but does not 
- *      add it to the prompt dialog.
+ *	add it to the prompt dialog.
  */
 static pdialog_prompt_struct *PDialogPromptNewNexus(
 	pdialog_prompt_type type,
@@ -783,7 +869,7 @@ static pdialog_prompt_struct *PDialogPromptNewNexus(
 	    return(p);
 
 	w = d->toplevel;
-	window = (w != NULL) ? w->window : NULL;      
+	window = (w != NULL) ? w->window : NULL;
 	style = gtk_widget_get_style(w); 
 
 	p->type = type;
@@ -828,9 +914,9 @@ static pdialog_prompt_struct *PDialogPromptNewNexus(
 	{
 	    GtkWidget *w2;
 	    const GtkTargetEntry dnd_tar_types[] = {
-{"text/plain",                          0,	0},
-{"text/uri-list",                       0,	1},
-{"STRING",                              0,	2}
+{"text/plain",				0,	0},
+{"text/uri-list",			0,	1},
+{"STRING",				0,	2}
 	    };
 	    /* Create by prompt type */
 	    switch(type)
@@ -865,7 +951,6 @@ static pdialog_prompt_struct *PDialogPromptNewNexus(
 		    gtk_entry_set_text(GTK_ENTRY(w), value);
 		    gtk_entry_set_position(GTK_ENTRY(w), -1);
 		}
-		GTK_WIDGET_SET_FLAGS(w, GTK_CAN_DEFAULT);
 		gtk_signal_connect(
 		    GTK_OBJECT(w), "activate",
 		    GTK_SIGNAL_FUNC(PDialogEntryEnterCB), d
@@ -899,9 +984,9 @@ static pdialog_prompt_struct *PDialogPromptNewNexus(
 		    w2,
 		    dnd_tar_types,
 		    sizeof(dnd_tar_types) / sizeof(GtkTargetEntry),
-		    GDK_ACTION_COPY,            /* Actions */
-		    GDK_ACTION_COPY,            /* Default action if same */
-		    GDK_ACTION_COPY,            /* Default action */
+		    GDK_ACTION_COPY,		/* Actions */
+		    GDK_ACTION_COPY,		/* Default action if same */
+		    GDK_ACTION_COPY,		/* Default action */
 		    PDialogPromptDragDataReceivedCB,
 		    w2
 		);
@@ -914,7 +999,6 @@ static pdialog_prompt_struct *PDialogPromptNewNexus(
 		    gtk_entry_set_text(GTK_ENTRY(w2), value);
 		    gtk_entry_set_position(GTK_ENTRY(w2), -1);
 		}
-		GTK_WIDGET_SET_FLAGS(w2, GTK_CAN_DEFAULT);
 		gtk_signal_connect(
 		    GTK_OBJECT(w2), "activate",
 		    GTK_SIGNAL_FUNC(PDialogEntryEnterCB), d
@@ -974,7 +1058,7 @@ static pdialog_prompt_struct *PDialogPromptNewNexus(
 		"Hojee"
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-		"Brouter"
+		"Naviguer"
 #endif
 #ifdef PROG_LANGUAGE_GERMAN
 		"Brausen"
@@ -1281,7 +1365,7 @@ gint PDialogInit(void)
 	    "Ayuda",
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
-	    "L'Aide",
+	    "Aide",
 #endif
 #ifdef PROG_LANGUAGE_GERMAN
 	    "Hilfe",
@@ -1313,7 +1397,7 @@ gint PDialogInit(void)
 }
 
 /*
- *      Sets the Dialog's style.
+ *	Sets the Dialog's style.
  */
 void PDialogSetStyle(GtkRcStyle *rc_style)
 {
@@ -1340,9 +1424,9 @@ void PDialogSetStyle(GtkRcStyle *rc_style)
 }
 
 /*
- *      Sets the Dialog to be a transient for the given GtkWindow w.
+ *	Sets the Dialog to be a transient for the given GtkWindow w.
  *
- *      If w is NULL then transient for will be unset.
+ *	If w is NULL then transient for will be unset.
  */
 void PDialogSetTransientFor(GtkWidget *w)
 {
@@ -1383,7 +1467,7 @@ void PDialogSetTransientFor(GtkWidget *w)
 }
 
 /*
- *      Checks if the Dialog is currently waiting for user input.
+ *	Checks if the Dialog is currently waiting for user input.
  */
 gboolean PDialogIsQuery(void)
 {
@@ -1392,8 +1476,8 @@ gboolean PDialogIsQuery(void)
 }
 
 /*
- *      Breaks the Dialog's user input query (if any) and causes the
- *      querying function to return CDIALOG_RESPONSE_NOT_AVAILABLE.
+ *	Breaks the Dialog's user input query (if any) and causes the
+ *	querying function to return CDIALOG_RESPONSE_NOT_AVAILABLE.
  */
 void PDialogBreakQuery(void)
 {
@@ -1489,7 +1573,7 @@ static pdialog_prompt_struct *PDialogAddPromptNexus(
  */
 void PDialogAddPromptLabel(const gchar *label)
 {
-	PDialogAddPromptNexus(   
+	PDialogAddPromptNexus(
 	    PDIALOG_PROMPT_TYPE_LABEL,
 	    NULL, NULL, label, FALSE, NULL, NULL
 	);
@@ -1511,7 +1595,7 @@ void PDialogAddPrompt(
 }
 
 /*
- *      Adds a prompt (with the value hidden for passwords) to the
+ *	Adds a prompt (with the value hidden for passwords) to the
  *	dialog.
  */
 void PDialogAddPromptPassword(
@@ -1544,7 +1628,7 @@ void PDialogAddPromptWithBrowse(
 }
 
 /*
- *      Adds a spin prompt to the dialog.
+ *	Adds a spin prompt to the dialog.
  */
 void PDialogAddPromptSpin(
 	const guint8 **icon_data,
@@ -1571,7 +1655,7 @@ void PDialogAddPromptSpin(
 }
 
 /*
- *      Adds a scale prompt to the dialog.
+ *	Adds a scale prompt to the dialog.
  */
 void PDialogAddPromptScale(
 	const guint8 **icon_data,
@@ -1633,18 +1717,20 @@ void PDialogAddPromptCombo(
 	    gtk_combo_set_use_arrows_always(combo, TRUE);
 	    if(list != NULL)
 		gtk_combo_set_popdown_strings(combo, list);
+	    if(value != NULL)
+		gtk_entry_set_text(entry, value);
 	}
 }
 
 /*
- *      Adds a radio prompt to the dialog.
+ *	Adds a radio prompt to the dialog.
  *
- *      The given glist is not modified by this function.
+ *	The specified glist is not modified by this function.
  */
 void PDialogAddPromptRadio(
 	const guint8 **icon_data,
 	const gchar *label,
-	GList *list,            /* List of radio button names */
+	GList *list,		/* List of radio button names */
 	gint start_num          /* Initial radio button to select */
 )
 {
@@ -1696,7 +1782,7 @@ void PDialogAddPromptRadio(
 }
 
 /*
- *      Adds a toggle prompt to the dialog.
+ *	Adds a toggle prompt to the dialog.
  */
 void PDialogAddPromptToggle(
 	const guint8 **icon_data,
@@ -1878,9 +1964,9 @@ void PDialogSetPromptValue(
 /*
  *	Sets the tip for the given prompt.
  *
- *      If any input is NULL then that value will be left unchanged.
+ *	If any input is NULL then that value will be left unchanged.
  *
- *      If prompt_num is -1 then the last prompt will be used.
+ *	If prompt_num is -1 then the last prompt will be used.
  */
 void PDialogSetPromptTip(gint prompt_num, const gchar *tip)
 {
@@ -1945,15 +2031,14 @@ void PDialogSetPromptTip(gint prompt_num, const gchar *tip)
 }
 
 /*
- *	Fetches the value of the specified prompt as a statically
- *	allocated string which must not be modified or deallocated.
+ *	Gets the value of the specified prompt as a string.
  *
  *	Can return NULL on error or if the prompt does not have a value
  *	to return.
  *
  *	If prompt_num is -1 then the last prompt will be used.
  */
-gchar *PDialogGetPromptValue(gint prompt_num)
+const gchar *PDialogGetPromptValue(gint prompt_num)
 {
 	pdialog_prompt_struct *p;
 	pdialog_struct *d = &pdialog;
@@ -2046,6 +2131,81 @@ gchar *PDialogGetPromptValue(gint prompt_num)
 	return(NULL);
 }
 
+/*
+ *	Sets the prompt to accept the tab key to complete paths.
+ *
+ *	If prompt_num is -1 then the last prompt will be used.
+ */
+void PDialogSetPromptCompletePath(gint prompt_num)
+{
+	pdialog_prompt_struct *p;
+	pdialog_struct *d = &pdialog;
+
+	if(prompt_num < 0)
+	    prompt_num = d->total_prompts - 1;
+
+	p = ((prompt_num >= 0) && (prompt_num < d->total_prompts)) ?
+	    d->prompt[prompt_num] : NULL;
+
+	if(p != NULL)
+	{
+	    GtkWidget *w = p->value;
+
+	    switch(p->type)
+	    {
+	      case PDIALOG_PROMPT_TYPE_LABEL:
+		break;
+
+	      case PDIALOG_PROMPT_TYPE_ENTRY:
+		if(GTK_IS_ENTRY(w))
+		{
+		    gtk_signal_connect(
+			GTK_OBJECT(w), "key_press_event",
+			GTK_SIGNAL_FUNC(PDialogEntryCompletePathKeyCB), d
+		    );
+		    gtk_signal_connect(
+			GTK_OBJECT(w), "key_release_event",
+			GTK_SIGNAL_FUNC(PDialogEntryCompletePathKeyCB), d
+		    );
+		}
+		break;
+
+	      case PDIALOG_PROMPT_TYPE_SPIN:
+		break;
+
+	      case PDIALOG_PROMPT_TYPE_SCALE:
+		break;
+
+	      case PDIALOG_PROMPT_TYPE_COMBO:
+		if(GTK_IS_COMBO(w))
+		{
+		    GtkCombo *combo = GTK_COMBO(w);
+		    w = combo->entry;
+		    if(GTK_IS_ENTRY(w))
+		    {
+			gtk_signal_connect(
+			    GTK_OBJECT(w), "key_press_event",
+			    GTK_SIGNAL_FUNC(PDialogEntryCompletePathKeyCB), d
+			);
+			gtk_signal_connect(
+			    GTK_OBJECT(w), "key_release_event",
+			    GTK_SIGNAL_FUNC(PDialogEntryCompletePathKeyCB), d
+			);
+		    }
+		}
+		break;
+
+	      case PDIALOG_PROMPT_TYPE_RADIO:
+		break;
+
+	      case PDIALOG_PROMPT_TYPE_TOGGLE:
+		break;
+
+	      case PDIALOG_PROMPT_TYPE_SEPARATOR:
+		break;
+	    }
+	}
+}
 
 /*
  *	Destroys all prompts on prompt dialog.
@@ -2087,6 +2247,7 @@ gchar **PDialogGetResponse(
 	gint *nvalues
 )
 {
+	gboolean set_focus = FALSE;
 	gint i;
 	GtkWidget *w;
 	pdialog_struct *d = &pdialog;
@@ -2160,12 +2321,12 @@ gchar **PDialogGetResponse(
 { if(w != NULL) gtk_widget_hide(w); }
 #define DO_DEFAULT_BUTTON	\
 { if(w != NULL) {		\
-/*  gtk_widget_grab_focus(w);	\
+/*gtk_widget_grab_focus(w);	\
   gtk_widget_grab_default(w); */\
 } }
 #define DO_UNDEFAULT_BUTTON	\
 { if(w != NULL) {		\
-/*  GTK_WIDGET_UNSET_FLAGS(w, GTK_HAS_DEFAULT); \
+/*GTK_WIDGET_UNSET_FLAGS(w, GTK_HAS_DEFAULT); \
   GTK_WIDGET_UNSET_FLAGS(w, GTK_RECEIVES_DEFAULT); */ \
 } }
 
@@ -2199,15 +2360,37 @@ gchar **PDialogGetResponse(
 	else
 	    DO_UNDEFAULT_BUTTON
 
-	/* Have first prompt to grab focus and grab default */
+	/* The first prompt should be in focus */
 	if(d->total_prompts > 0)
 	{
 	    pdialog_prompt_struct *p = d->prompt[0];
 	    w = (p != NULL) ? p->value : NULL;
-	    if(w != NULL)
+	    if((w != NULL) ? GTK_WIDGET_CAN_FOCUS(w) : FALSE)
 	    {
 		gtk_widget_grab_focus(w);
-		gtk_widget_grab_default(w);
+		set_focus = TRUE;
+	    }
+	}
+
+	if(!set_focus)
+	{
+	    if(default_button & PDIALOG_BTNFLAG_SUBMIT)
+	    {
+		w = d->submit_btn;
+		if(w != NULL)
+		{
+		    gtk_widget_grab_focus(w);
+		    set_focus = TRUE;
+		}
+	    }
+	    else if(default_button & PDIALOG_BTNFLAG_CANCEL)
+	    {
+		w = d->cancel_btn;
+		if(w != NULL)
+		{
+		    gtk_widget_grab_focus(w);
+		    set_focus = TRUE;
+		}
 	    }
 	}
 
@@ -2219,8 +2402,9 @@ gchar **PDialogGetResponse(
 	/* Widget sizes may have changed due to new values est, queue
 	 * resize for the toplevel
 	 */
-	if(d->toplevel != NULL)
-	    gtk_widget_queue_resize(d->toplevel);
+	w = d->toplevel;
+	if(w != NULL)
+	    gtk_widget_queue_resize(w);
 
 	/* Map dialog */
 	PDialogMap();
@@ -2256,7 +2440,7 @@ gchar **PDialogGetResponseIconData(
 	const gchar *title,
 	const gchar *message,
 	const gchar *explaination,
-	guint8 **icon_data,       
+	guint8 **icon_data,
 	const gchar *submit_label,
 	const gchar *cancel_label,
 	pdialog_btn_flags show_buttons,		/* Any of PDIALOG_BTNFLAG_* */
@@ -2336,12 +2520,12 @@ gchar **PDialogGetResponseIconData(
 { if(w != NULL) gtk_widget_hide(w); }
 #define DO_DEFAULT_BUTTON       \
 { if(w != NULL) {               \
-/*  gtk_widget_grab_focus(w);   \
+/*gtk_widget_grab_focus(w);	\
   gtk_widget_grab_default(w); */\
 } }
 #define DO_UNDEFAULT_BUTTON     \
 { if(w != NULL) {               \
-/*  GTK_WIDGET_UNSET_FLAGS(w, GTK_HAS_DEFAULT); \
+/*GTK_WIDGET_UNSET_FLAGS(w, GTK_HAS_DEFAULT); \
   GTK_WIDGET_UNSET_FLAGS(w, GTK_RECEIVES_DEFAULT); */ \
 } }
 
@@ -2375,16 +2559,13 @@ gchar **PDialogGetResponseIconData(
 	else
 	    DO_UNDEFAULT_BUTTON
 
-	/* Have first prompt to grab focus and grab default */
+	/* Have first prompt to grab focus */
 	if(d->total_prompts > 0)
 	{
 	    pdialog_prompt_struct *p = d->prompt[0];
 	    w = (p != NULL) ? p->value : NULL;
 	    if(w != NULL)
-	    {
 		gtk_widget_grab_focus(w);
-		gtk_widget_grab_default(w);
-	    }
 	}    
 
 #undef DO_MAP_BUTTON
@@ -2409,7 +2590,7 @@ gchar **PDialogGetResponseIconData(
 	PDialogUnmap();
 
 	/* Break out of any subsequent main loops */
-	while(d->block_loop_level > 0)              
+	while(d->block_loop_level > 0)
 	{
 	    gtk_main_quit();
 	    d->block_loop_level--;
@@ -2420,7 +2601,7 @@ gchar **PDialogGetResponseIconData(
 	if(nvalues != NULL)
 	    *nvalues = d->response_nvals;
 					 
-	return(d->response_val);         
+	return(d->response_val);
 }
 
 
